@@ -28,12 +28,15 @@
 //
 ///////////////////////////////////////////////////////////////////////////
 
-
-#include "Viewer.h"
+#include <openvdb_viewer/Viewer.h>
 #include <iostream>
 #include <string>
 #include <vector>
 #include <exception>
+#ifdef DWA_OPENVDB
+#include <logging_base/logging.h>
+#include <usagetrack.h>
+#endif
 
 
 void
@@ -76,8 +79,13 @@ usage(const char* progName, int status)
 
 
 int
-main(int argc, char * const argv[])
+main(int argc, char *argv[])
 {
+#ifdef DWA_OPENVDB
+    USAGETRACK_report_basic_tool_usage(argc, argv, /*duration=*/0);
+    logging_base::configure(argc, argv);
+#endif
+
     const char* progName = argv[0];
     if (const char* ptr = ::strrchr(progName, '/')) progName = ptr + 1;
 
@@ -105,7 +113,7 @@ main(int argc, char * const argv[])
             }
         }
 
-        Viewer::init(progName, printDebugInfo);
+        openvdb_viewer::Viewer viewer = openvdb_viewer::init(progName, printDebugInfo);
 
         const size_t numFiles = filenames.size();
         if (numFiles == 0) usage(progName, EXIT_FAILURE);
@@ -137,7 +145,7 @@ main(int argc, char * const argv[])
             }
         }
 
-        Viewer::view(allGrids);
+        viewer.view(allGrids);
 
     } catch (const char* s) {
         OPENVDB_LOG_ERROR(progName << ": " << s);
