@@ -54,6 +54,7 @@ struct StreamState
     static const long MAGIC_NUMBER;
 
     StreamState();
+    ~StreamState();
 
     int magicNumber;
     int fileVersion;
@@ -125,6 +126,14 @@ StreamState::StreamState(): magicNumber(std::ios_base::xalloc())
 }
 
 
+StreamState::~StreamState()
+{
+    // Ensure that this StreamState struct can no longer be accessed.
+    std::cout.iword(magicNumber) = 0;
+    std::cout.pword(magicNumber) = NULL;
+}
+
+
 ////////////////////////////////////////
 
 
@@ -146,6 +155,13 @@ Archive::~Archive()
 }
 
 
+boost::shared_ptr<Archive>
+Archive::copy() const
+{
+    return boost::shared_ptr<Archive>(new Archive(*this));
+}
+
+
 ////////////////////////////////////////
 
 
@@ -596,9 +612,12 @@ Archive::write(std::ostream& os, const GridCPtrVec& grids, bool seekable,
     for (GridCPtrVecCIter i = grids.begin(), e = grids.end(); i != e; ++i) {
         if (const GridBase::ConstPtr& grid = *i) {
 
-            // Ensure that the grid's descriptor has a unique grid name.
+            // Ensure that the grid's descriptor has a unique grid name, by appending
+            // a number to it if a grid with the same name was already written.
+            // Always add a number if the grid name is empty, so that the grid can be
+            // properly identified as an instance parent, if necessary.
             std::string name = grid->getName();
-            for (int n = 1; uniqueNames.find(name) != uniqueNames.end(); ++n) {
+            for (int n = 1; name.empty() || uniqueNames.find(name) != uniqueNames.end(); ++n) {
                 name = GridDescriptor::addSuffix(grid->getName(), n);
             }
             uniqueNames.insert(name);
