@@ -37,6 +37,7 @@
 #include <string>
 #include <boost/uuid/uuid.hpp>
 #include <boost/cstdint.hpp>
+#include <boost/shared_ptr.hpp>
 #include <openvdb/Grid.h>
 #include <openvdb/metadata/MetaMap.h>
 #include <openvdb/version.h> // for VersionId
@@ -117,6 +118,9 @@ public:
     Archive();
     virtual ~Archive();
 
+    /// @brief Return a copy of this archive.
+    virtual boost::shared_ptr<Archive> copy() const;
+
     /// @brief Return the UUID that was most recently written (or read,
     /// if no UUID has been written yet).
     std::string getUniqueTag() const;
@@ -163,6 +167,9 @@ public:
     /// bounding box, etc.) should be computed and written as grid metadata.
     void setGridStatsMetadataEnabled(bool b) { mEnableGridStats = b; }
 
+    /// @brief Write the grids in the given container to this archive's output stream.
+    virtual void write(const GridCPtrVec&, const MetaMap& = MetaMap()) const {}
+
 protected:
     /// @brief Return @c true if the input stream contains grid offsets
     /// that allow for random access or partial reading.
@@ -198,7 +205,7 @@ protected:
     void setWriteGridStatsMetadata(std::ostream&);
 
     /// Read in and return the number of grids on the input stream.
-    static int readGridCount(std::istream&);
+    static int32_t readGridCount(std::istream&);
 
     /// Populate the given grid from the input stream.
     static void readGrid(GridBase::Ptr, const GridDescriptor&, std::istream&);
@@ -242,7 +249,7 @@ private:
     /// The version of the library that was used to create the file that was read
     VersionId mLibraryVersion;
     /// 16-byte (128-bit) UUID
-    mutable boost::uuids::uuid mUuid;// needs to mutable since writeHeader is const!
+    mutable boost::uuids::uuid mUuid;// needs to be mutable since writeHeader is const!
     /// Flag indicating whether the input stream contains grid offsets
     /// and therefore supports partial reading
     bool mInputHasGridOffsets;
