@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -357,7 +357,7 @@ public:
                        );
     } // trivial
 
-    /// @brief Returns m, where \f$m_{i,j} *= scalar\f$ for \f$i, j \in [0, 3]\f$
+    /// Return m, where \f$m_{i,j} *= scalar\f$ for \f$i, j \in [0, 3]\f$
     template <typename S>
     const Mat4<T>& operator*=(S scalar)
     {
@@ -441,7 +441,7 @@ public:
         return *this;
     }
 
-    /// @brief Returns m, where \f$m_{i,j} = \sum_{k} m0_{i,k}*m1_{k,j}\f$ for \f$i, j \in [0, 3]\f$
+    /// Return m, where \f$m_{i,j} = \sum_{k} m0_{i,k}*m1_{k,j}\f$ for \f$i, j \in [0, 3]\f$
     template <typename S>
     const Mat4<T> &operator*=(const Mat4<S> &m1)
     {
@@ -456,17 +456,17 @@ public:
                                               s0[i4+1] * s1[ 4] +
                                               s0[i4+2] * s1[ 8] +
                                               s0[i4+3] * s1[12]);
-            
+
             MyBase::mm[i4+1] = static_cast<T>(s0[i4+0] * s1[ 1] +
                                               s0[i4+1] * s1[ 5] +
                                               s0[i4+2] * s1[ 9] +
                                               s0[i4+3] * s1[13]);
-            
+
             MyBase::mm[i4+2] = static_cast<T>(s0[i4+0] * s1[ 2] +
                                               s0[i4+1] * s1[ 6] +
                                               s0[i4+2] * s1[10] +
                                               s0[i4+3] * s1[14]);
-            
+
             MyBase::mm[i4+3] = static_cast<T>(s0[i4+0] * s1[ 3] +
                                               s0[i4+1] * s1[ 7] +
                                               s0[i4+2] * s1[11] +
@@ -485,7 +485,7 @@ public:
                        MyBase::mm[ 3], MyBase::mm[ 7], MyBase::mm[11], MyBase::mm[15]
                        );
     }
-    
+
 
     /// @return inverse of this
     /// @throw ArithmeticError if singular
@@ -711,27 +711,27 @@ public:
         MyBase::mm[14] = v.z();
         MyBase::mm[15] = 1;
     }
-    
+
     /// Left multiples by the specified translation, i.e.  Trans * (*this)
     template <typename T0>
-    void preTranslate(const Vec3<T0>& tr) 
+    void preTranslate(const Vec3<T0>& tr)
     {
         Vec3<T> tmp(tr.x(), tr.y(), tr.z());
         Mat4<T> Tr = Mat4<T>::translation(tmp);
 
-        *this =  Tr * (*this);  
-        
+        *this =  Tr * (*this);
+
     }
 
     /// Right multiplies by the specified translation matrix, i.e. (*this) * Trans
     template <typename T0>
-    void postTranslate(const Vec3<T0>& tr) 
+    void postTranslate(const Vec3<T0>& tr)
     {
         Vec3<T> tmp(tr.x(), tr.y(), tr.z());
         Mat4<T> Tr = Mat4<T>::translation(tmp);
 
-        *this = (*this) * Tr;  
-        
+        *this = (*this) * Tr;
+
     }
 
 
@@ -771,26 +771,26 @@ public:
     template <typename T0>
     void postScale(const Vec3<T0>& v)
     {
-       
+
         MyBase::mm[ 0] *= v.x();
-        MyBase::mm[ 1] *= v.x();
-        MyBase::mm[ 2] *= v.x();
-       
-        MyBase::mm[ 4] *= v.y();
+        MyBase::mm[ 1] *= v.y();
+        MyBase::mm[ 2] *= v.z();
+
+        MyBase::mm[ 4] *= v.x();
         MyBase::mm[ 5] *= v.y();
-        MyBase::mm[ 6] *= v.y();
-       
-        MyBase::mm[ 8] *= v.z();
-        MyBase::mm[ 9] *= v.z();
+        MyBase::mm[ 6] *= v.z();
+
+        MyBase::mm[ 8] *= v.x();
+        MyBase::mm[ 9] *= v.y();
         MyBase::mm[10] *= v.z();
-       
+
         MyBase::mm[12] *= v.x();
         MyBase::mm[13] *= v.y();
         MyBase::mm[14] *= v.z();
 
     }
 
-    
+
     /// @brief Sets the matrix to a rotation about the given axis.
     /// @param axis The axis (one of X, Y, Z) to rotate about.
     /// @param angle The rotation angle, in radians.
@@ -809,7 +809,7 @@ public:
     /// @brief Left multiplies by a rotation clock-wiseabout the given axis into this matrix.
     /// @param axis The axis (one of X, Y, Z) of rotation.
     /// @param angle The clock-wise rotation angle, in radians.
-    void preRotate(Axis axis, T angle) 
+    void preRotate(Axis axis, T angle)
     {
         T c = static_cast<T>(cos(angle));
         T s = -static_cast<T>(sin(angle)); // the "-" makes it clockwise
@@ -818,18 +818,18 @@ public:
         case X_AXIS:
             {
                 T a4, a5, a6, a7;
-                
+
                 a4 = c * MyBase::mm[ 4] - s * MyBase::mm[ 8];
                 a5 = c * MyBase::mm[ 5] - s * MyBase::mm[ 9];
                 a6 = c * MyBase::mm[ 6] - s * MyBase::mm[10];
                 a7 = c * MyBase::mm[ 7] - s * MyBase::mm[11];
-                
-                
+
+
                 MyBase::mm[ 8] = s * MyBase::mm[ 4] + c * MyBase::mm[ 8];
                 MyBase::mm[ 9] = s * MyBase::mm[ 5] + c * MyBase::mm[ 9];
                 MyBase::mm[10] = s * MyBase::mm[ 6] + c * MyBase::mm[10];
                 MyBase::mm[11] = s * MyBase::mm[ 7] + c * MyBase::mm[11];
-                
+
                 MyBase::mm[ 4] = a4;
                 MyBase::mm[ 5] = a5;
                 MyBase::mm[ 6] = a6;
@@ -840,18 +840,18 @@ public:
         case Y_AXIS:
             {
                 T a0, a1, a2, a3;
-                
+
                 a0 = c * MyBase::mm[ 0] + s * MyBase::mm[ 8];
                 a1 = c * MyBase::mm[ 1] + s * MyBase::mm[ 9];
                 a2 = c * MyBase::mm[ 2] + s * MyBase::mm[10];
                 a3 = c * MyBase::mm[ 3] + s * MyBase::mm[11];
-                
+
                 MyBase::mm[ 8] = -s * MyBase::mm[ 0] + c * MyBase::mm[ 8];
                 MyBase::mm[ 9] = -s * MyBase::mm[ 1] + c * MyBase::mm[ 9];
                 MyBase::mm[10] = -s * MyBase::mm[ 2] + c * MyBase::mm[10];
                 MyBase::mm[11] = -s * MyBase::mm[ 3] + c * MyBase::mm[11];
-                
-                
+
+
                 MyBase::mm[ 0] = a0;
                 MyBase::mm[ 1] = a1;
                 MyBase::mm[ 2] = a2;
@@ -862,17 +862,17 @@ public:
         case Z_AXIS:
             {
                 T a0, a1, a2, a3;
- 
+
                 a0 = c * MyBase::mm[ 0] - s * MyBase::mm[ 4];
                 a1 = c * MyBase::mm[ 1] - s * MyBase::mm[ 5];
                 a2 = c * MyBase::mm[ 2] - s * MyBase::mm[ 6];
                 a3 = c * MyBase::mm[ 3] - s * MyBase::mm[ 7];
-                
+
                 MyBase::mm[ 4] = s * MyBase::mm[ 0] + c * MyBase::mm[ 4];
                 MyBase::mm[ 5] = s * MyBase::mm[ 1] + c * MyBase::mm[ 5];
                 MyBase::mm[ 6] = s * MyBase::mm[ 2] + c * MyBase::mm[ 6];
                 MyBase::mm[ 7] = s * MyBase::mm[ 3] + c * MyBase::mm[ 7];
-                
+
                 MyBase::mm[ 0] = a0;
                 MyBase::mm[ 1] = a1;
                 MyBase::mm[ 2] = a2;
@@ -889,24 +889,24 @@ public:
     /// @brief Right multiplies by a rotation clock-wiseabout the given axis into this matrix.
     /// @param axis The axis (one of X, Y, Z) of rotation.
     /// @param angle The clock-wise rotation angle, in radians.
-    void postRotate(Axis axis, T angle) 
+    void postRotate(Axis axis, T angle)
     {
         T c = static_cast<T>(cos(angle));
         T s = -static_cast<T>(sin(angle)); // the "-" makes it clockwise
 
-       
+
 
         switch (axis) {
         case X_AXIS:
             {
                 T a2, a6, a10, a14;
-            
+
                 a2  = c * MyBase::mm[ 2] - s * MyBase::mm[ 1];
                 a6  = c * MyBase::mm[ 6] - s * MyBase::mm[ 5];
                 a10 = c * MyBase::mm[10] - s * MyBase::mm[ 9];
                 a14 = c * MyBase::mm[14] - s * MyBase::mm[13];
-                
-                
+
+
                 MyBase::mm[ 1] = c * MyBase::mm[ 1] + s * MyBase::mm[ 2];
                 MyBase::mm[ 5] = c * MyBase::mm[ 5] + s * MyBase::mm[ 6];
                 MyBase::mm[ 9] = c * MyBase::mm[ 9] + s * MyBase::mm[10];
@@ -922,17 +922,17 @@ public:
         case Y_AXIS:
             {
                 T a2, a6, a10, a14;
-                
+
                 a2  = c * MyBase::mm[ 2] + s * MyBase::mm[ 0];
                 a6  = c * MyBase::mm[ 6] + s * MyBase::mm[ 4];
                 a10 = c * MyBase::mm[10] + s * MyBase::mm[ 8];
                 a14 = c * MyBase::mm[14] + s * MyBase::mm[12];
-                
+
                 MyBase::mm[ 0] = c * MyBase::mm[ 0] - s * MyBase::mm[ 2];
                 MyBase::mm[ 4] = c * MyBase::mm[ 4] - s * MyBase::mm[ 6];
                 MyBase::mm[ 8] = c * MyBase::mm[ 8] - s * MyBase::mm[10];
                 MyBase::mm[12] = c * MyBase::mm[12] - s * MyBase::mm[14];
-                    
+
                 MyBase::mm[ 2] = a2;
                 MyBase::mm[ 6] = a6;
                 MyBase::mm[10] = a10;
@@ -943,7 +943,7 @@ public:
         case Z_AXIS:
             {
                 T a1, a5, a9, a13;
-                
+
                 a1  = c * MyBase::mm[ 1] - s * MyBase::mm[ 0];
                 a5  = c * MyBase::mm[ 5] - s * MyBase::mm[ 4];
                 a9  = c * MyBase::mm[ 9] - s * MyBase::mm[ 8];
@@ -953,12 +953,12 @@ public:
                 MyBase::mm[ 4] = c * MyBase::mm[ 4] + s * MyBase::mm[ 5];
                 MyBase::mm[ 8] = c * MyBase::mm[ 8] + s * MyBase::mm[ 9];
                 MyBase::mm[12] = c * MyBase::mm[12] + s * MyBase::mm[13];
-                
+
                 MyBase::mm[ 1] = a1;
                 MyBase::mm[ 5] = a5;
                 MyBase::mm[ 9] = a9;
                 MyBase::mm[13] = a13;
-         
+
             }
             break;
 
@@ -983,22 +983,22 @@ public:
     {
         int index0 = static_cast<int>(axis0);
         int index1 = static_cast<int>(axis1);
-        
-        // to row "index1" add a multiple of the index0 row 
+
+        // to row "index1" add a multiple of the index0 row
         MyBase::mm[index1 * 4 + 0] += shear * MyBase::mm[index0 * 4 + 0];
         MyBase::mm[index1 * 4 + 1] += shear * MyBase::mm[index0 * 4 + 1];
         MyBase::mm[index1 * 4 + 2] += shear * MyBase::mm[index0 * 4 + 2];
         MyBase::mm[index1 * 4 + 3] += shear * MyBase::mm[index0 * 4 + 3];
     }
-    
-    
+
+
     /// @brief Right multiplies a shearing transformation into the matrix.
     /// @see setToShear
     void postShear(Axis axis0, Axis axis1, T shear)
     {
         int index0 = static_cast<int>(axis0);
         int index1 = static_cast<int>(axis1);
-        
+
         // to collumn "index0" add a multiple of the index1 row
         MyBase::mm[index0 +  0] += shear * MyBase::mm[index1 +  0];
         MyBase::mm[index0 +  4] += shear * MyBase::mm[index1 +  4];
@@ -1006,7 +1006,7 @@ public:
         MyBase::mm[index0 + 12] += shear * MyBase::mm[index1 + 12];
 
     }
-    
+
     /// Transform a Vec4 by post-multiplication.
     template<typename T0>
     Vec4<T0> transform(const Vec4<T0> &v) const
@@ -1088,246 +1088,6 @@ public:
     }
 
 
-
-    /// For certain legacy APIs, we need to treat the contents as a pointer.
-    /// @remark This was originally designed as an "operator T*()", but that
-    /// automatic pointer cast would be applied in unexpected cases, such as
-    /// std::min/max.
-#ifdef  ALLOW_CAST_TO_POINTER
-    /// Cast to T*
-OPENVDB_DEPRECATED    operator T*()
-#if (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
-            __attribute__ ((deprecated))
-#endif
-            {return MyBase::mm;}
-OPENVDB_DEPRECATED    operator const T*() const
-#if (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
-            __attribute__ ((deprecated))
-#endif
-            {return MyBase::mm;}
-#endif
-
-
-
-    /// @brief Accumulates a shearing transformation into the matrix.
-    /// @see setToShear
-OPENVDB_DEPRECATED void accumShear(Axis axis0, Axis axis1, T shear)
-    {
-        int index0 = static_cast<int>(axis0);
-        int index1 = static_cast<int>(axis1);
-
-        MyBase::mm[index1 * 4 + 0] += shear * MyBase::mm[index0 * 4    ];
-        MyBase::mm[index1 * 4 + 1] += shear * MyBase::mm[index0 * 4 + 1];
-        MyBase::mm[index1 * 4 + 2] += shear * MyBase::mm[index0 * 4 + 2];
-        MyBase::mm[index1 * 4 + 3] += shear * MyBase::mm[index0 * 4 + 3];
-    }
-
-
-    /// Accumulates a scaling of v into the matrix
-    template <typename T0>
-OPENVDB_DEPRECATED void accumScale(const Vec3<T0>& v)
-    {
-        preScale(v);
-    }
-
-
-    /// Accumulates a translation of v into the matrix
-    /// this assumes that mm[15] == 1.
-    template <typename T0>
-OPENVDB_DEPRECATED void accumTranslation(const Vec3<T0>& v)
-    {
-        this->mm[12] += v.dot(Vec3<T>(this->mm[ 0], this->mm[ 4], this->mm[ 8]));
-        this->mm[13] += v.dot(Vec3<T>(this->mm[ 1], this->mm[ 5], this->mm[ 9]));
-        this->mm[14] += v.dot(Vec3<T>(this->mm[ 2], this->mm[ 6], this->mm[10]));
-        this->mm[15] += v.dot(Vec3<T>(this->mm[ 3], this->mm[ 7], this->mm[11]));
-    }
-
-
-    /// @brief Accumulates a rotation about the given axis into this matrix.
-    /// @param axis The axis (one of X, Y, Z) of rotation.
-    /// @param angle The rotation angle, in radians.
-OPENVDB_DEPRECATED void accumRotation(Axis axis, T angle)
-    {
-        T c = static_cast<T>(cos(angle));
-        T s = static_cast<T>(sin(angle));
-
-        T a10, a11, a12;
-        T a00, a01, a02;
-
-        switch (axis) {
-        case X_AXIS:
-            a10 = c*this->mm[ 4] + s*this->mm[ 8];
-            a11 = c*this->mm[ 5] + s*this->mm[ 9];
-            a12 = c*this->mm[ 6] + s*this->mm[10];
-
-            this->mm[ 8]  = -s*this->mm[ 4] + c*this->mm[ 8];
-            this->mm[ 9]  = -s*this->mm[ 5] + c*this->mm[ 9];
-            this->mm[10] = -s*this->mm[ 6] + c*this->mm[10];
-
-            this->mm[ 4] = a10;
-            this->mm[ 5] = a11;
-            this->mm[ 6] = a12;
-
-            break;
-
-        case Y_AXIS:
-            a00 = c*this->mm[ 0] - s*this->mm[ 8];
-            a01 = c*this->mm[ 1] - s*this->mm[ 9];
-            a02 = c*this->mm[ 2] - s*this->mm[10];
-
-            this->mm[ 8]  = s*this->mm[ 0] + c*this->mm[ 8];
-            this->mm[ 9]  = s*this->mm[ 1] + c*this->mm[ 9];
-            this->mm[10] = s*this->mm[ 2] + c*this->mm[10];
-
-            this->mm[ 0] = a00;
-            this->mm[ 1] = a01;
-            this->mm[ 2] = a02;
-
-            break;
-
-        case Z_AXIS:
-            a00 = c*this->mm[ 0] + s*this->mm[ 4];
-            a01 = c*this->mm[ 1] + s*this->mm[ 5];
-            a02 = c*this->mm[ 2] + s*this->mm[ 6];
-
-            this->mm[ 4] = -s*this->mm[ 0] + c*this->mm[ 4];
-            this->mm[ 5] = -s*this->mm[ 1] + c*this->mm[ 5];
-            this->mm[ 6] = -s*this->mm[ 2] + c*this->mm[ 6];
-
-            this->mm[ 0] = a00;
-            this->mm[ 1] = a01;
-            this->mm[ 2] = a02;
-
-            break;
-
-        default:
-            assert(axis==X_AXIS || axis==Y_AXIS || axis==Z_AXIS);
-        }
-    }
-
-
-    /// this = m1 + m2
-    /// @note m1, m2 and this need not be distinct objects, e.g., m1.add(m1, m2)
-    template <typename T0, typename T1>
-OPENVDB_DEPRECATED const Mat4& add(const Mat4<T0> &m1, const Mat4<T1> &m2)
-    {
-        const T0 *src1 = m1.asPointer();
-        const T1 *src2 = m2.asPointer();
-        this->mm[ 0] = src1[0] + src2[0];
-        this->mm[ 1] = src1[1] + src2[1];
-        this->mm[ 2] = src1[2] + src2[2];
-        this->mm[ 3] = src1[3] + src2[3];
-
-        this->mm[ 4] = src1[4] + src2[4];
-        this->mm[ 5] = src1[5] + src2[5];
-        this->mm[ 6] = src1[6] + src2[6];
-        this->mm[ 7] = src1[7] + src2[7];
-
-        this->mm[ 8] = src1[8] + src2[8];
-        this->mm[ 9] = src1[9] + src2[9];
-        this->mm[10] = src1[10] + src2[10];
-        this->mm[11] = src1[11] + src2[11];
-
-        this->mm[12] = src1[12] + src2[12];
-        this->mm[13] = src1[13] + src2[13];
-        this->mm[14] = src1[14] + src2[14];
-        this->mm[15] = src1[15] + src2[15];
-        return *this;
-    }
-
-    /// this = m1 - m2
-    /// m1 and m2 need not be distinct objects than "this", e.g. m1.sub(m1,m2);
-    template <typename T0, typename T1>
-OPENVDB_DEPRECATED const Mat4& sub(const Mat4<T0> &m1, const Mat4<T1> &m2)
-    {
-        const T0 *src1 = m1.asPointer();
-        const T1 *src2 = m2.asPointer();
-        this->mm[ 0] = src1[0] - src2[0];
-        this->mm[ 1] = src1[1] - src2[1];
-        this->mm[ 2] = src1[2] - src2[2];
-        this->mm[ 3] = src1[3] - src2[3];
-
-        this->mm[ 4] = src1[4] - src2[4];
-        this->mm[ 5] = src1[5] - src2[5];
-        this->mm[ 6] = src1[6] - src2[6];
-        this->mm[ 7] = src1[7] - src2[7];
-
-        this->mm[ 8] = src1[8] - src2[8];
-        this->mm[ 9] = src1[9] - src2[9];
-        this->mm[10] = src1[10] - src2[10];
-        this->mm[11] = src1[11] - src2[11];
-
-        this->mm[12] = src1[12] - src2[12];
-        this->mm[13] = src1[13] - src2[13];
-        this->mm[14] = src1[14] - src2[14];
-        this->mm[15] = src1[15] - src2[15];
-        return *this;
-    }
-
-    /// this = m * scalar
-    template <typename T0, typename T1>
-OPENVDB_DEPRECATED const Mat4& scale(T0 scalar, const Mat4<T1> &m)
-    {
-        const T1 *src = m.asPointer();
-        this->mm[ 0] = scalar * src[0];
-        this->mm[ 1] = scalar * src[1];
-        this->mm[ 2] = scalar * src[2];
-        this->mm[ 3] = scalar * src[3];
-
-        this->mm[ 4] = scalar * src[4];
-        this->mm[ 5] = scalar * src[5];
-        this->mm[ 6] = scalar * src[6];
-        this->mm[ 7] = scalar * src[7];
-
-        this->mm[ 8] = scalar * src[8];
-        this->mm[ 9] = scalar * src[9];
-        this->mm[10] = scalar * src[10];
-        this->mm[11] = scalar * src[11];
-
-        this->mm[12] = scalar * src[12];
-        this->mm[13] = scalar * src[13];
-        this->mm[14] = scalar * src[14];
-        this->mm[15] = scalar * src[15];
-        return *this;
-    }
-
-    /// this = m1 * m2
-    /// @note m1 and m2 must not be the same object as this.
-    template <typename T0, typename T1>
-OPENVDB_DEPRECATED const Mat4& mult(const Mat4<T0> &m1, const Mat4<T1> &m2)
-    {
-        register int  i, i4;
-        const T0 *src1 = m1.asPointer();
-        const T1 *src2 = m2.asPointer();
-
-        for (i = 0; i < 4; i++) {
-            i4 = 4 * i;
-            this->mm[i4+0] = static_cast<T>(src1[i4+0]*src2[4*0+0] +
-                                            src1[i4+1]*src2[4*1+0] +
-                                            src1[i4+2]*src2[4*2+0] +
-                                            src1[i4+3]*src2[4*3+0]);
-            
-            this->mm[i4+1] = static_cast<T>(src1[i4+0]*src2[4*0+1] +
-                                            src1[i4+1]*src2[4*1+1] +
-                                            src1[i4+2]*src2[4*2+1] +
-                                            src1[i4+3]*src2[4*3+1]);
-            
-            this->mm[i4+2] = static_cast<T>(src1[i4+0]*src2[4*0+2] +
-                                            src1[i4+1]*src2[4*1+2] +
-                                            src1[i4+2]*src2[4*2+2] +
-                                            src1[i4+3]*src2[4*3+2]);
-            
-            this->mm[i4+3] = static_cast<T>(src1[i4+0]*src2[4*0+3] +
-                                            src1[i4+1]*src2[4*1+3] +
-                                            src1[i4+2]*src2[4*2+3] +
-                                            src1[i4+3]*src2[4*3+3]);
-        }
-
-        return *this;
-    }
-
-
-
 private:
     bool invert(Mat4<T> &inverse, T tolerance) const;
 
@@ -1582,7 +1342,7 @@ template <typename T>
 inline bool hasTranslation(const Mat4<T>& m) {
     return (m.row(3) != Vec4<T>(0, 0, 0, 1));
 }
-    
+
 
 typedef Mat4<float>  Mat4s;
 typedef Mat4<double> Mat4d;
@@ -1596,20 +1356,14 @@ typedef Mat4s    Mat4f;
 } // namespace math
 
 
-template <> inline math::Mat4s zeroVal< math::Mat4s >()
-{
-    return math::Mat4s::identity();
-}
-template <> inline math::Mat4d zeroVal< math::Mat4d >()
-{
-    return math::Mat4d::identity();
-}
+template<> inline math::Mat4s zeroVal<math::Mat4s>() { return math::Mat4s::identity(); }
+template<> inline math::Mat4d zeroVal<math::Mat4d>() { return math::Mat4d::identity(); }
 
 } // namespace OPENVDB_VERSION_NAME
 } // namespace openvdb
 
 #endif // OPENVDB_UTIL_MAT4_H_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
