@@ -71,7 +71,9 @@ template<typename ScalarGridType> struct ScalarToVectorConverter {
 /// @brief Compute the Closest-Point Transform (CPT) from a distance field.
 /// @return a new vector-valued grid with the same numerical precision as the input grid
 ///     (for example, if the input grid is a DoubleGrid, the output grid will be a Vec3DGrid)
-///
+/// When a mask grid is specified, the solution is calculated only in the intersection of
+/// the mask active topology and the input active topology independent of the 
+/// transforms associated with either grid.
 /// @note The current implementation assumes all the input distance values
 /// are represented by leaf voxels and not tiles.  This is true for all
 /// narrow-band level sets, which this class was originally developed for.
@@ -79,6 +81,11 @@ template<typename ScalarGridType> struct ScalarToVectorConverter {
 template<typename GridType, typename InterruptT> inline
 typename ScalarToVectorConverter<GridType>::Type::Ptr
 cpt(const GridType& grid, bool threaded, InterruptT* interrupt);
+
+template<typename GridType, typename MaskT, typename InterruptT> inline
+typename ScalarToVectorConverter<GridType>::Type::Ptr
+cpt(const GridType& grid, const MaskT& mask, bool threaded, InterruptT* interrupt);
+
     
 template<typename GridType> inline
 typename ScalarToVectorConverter<GridType>::Type::Ptr
@@ -87,12 +94,27 @@ cpt(const GridType& grid, bool threaded = true)
     return cpt<GridType, util::NullInterrupter>(grid, threaded, NULL);
 }
 
+    
+template<typename GridType, typename MaskT> inline
+typename ScalarToVectorConverter<GridType>::Type::Ptr
+cpt(const GridType& grid, const MaskT& mask, bool threaded = true)
+{
+    return cpt<GridType, MaskT, util::NullInterrupter>(grid, mask, threaded, NULL);
+}
+
 
 /// @brief Compute the curl of the given vector-valued grid.
 /// @return a new vector-valued grid
+/// When a mask grid is specified, the solution is calculated only in the intersection of
+/// the mask active topology and the input active topology independent of the 
+/// transforms associated with either grid.
 template<typename GridType, typename InterruptT> inline
 typename GridType::Ptr
 curl(const GridType& grid, bool threaded, InterruptT* interrupt);
+
+template<typename GridType, typename MaskT, typename InterruptT> inline
+typename GridType::Ptr
+curl(const GridType& grid, const MaskT& mask, bool threaded, InterruptT* interrupt);
     
 template<typename GridType> inline
 typename GridType::Ptr
@@ -101,12 +123,26 @@ curl(const GridType& grid, bool threaded = true)
     return curl<GridType, util::NullInterrupter>(grid, threaded, NULL);
 }
 
+template<typename GridType, typename MaskT> inline
+typename GridType::Ptr
+curl(const GridType& grid, const MaskT& mask, bool threaded = true)
+{
+    return curl<GridType, MaskT, util::NullInterrupter>(grid, mask, threaded, NULL);
+}
+
 /// @brief Compute the divergence of the given vector-valued grid.
 /// @return a new scalar-valued grid with the same numerical precision as the input grid
 ///     (for example, if the input grid is a Vec3DGrid, the output grid will be a DoubleGrid)
+/// When a mask grid is specified, the solution is calculated only in the intersection of
+/// the mask active topology and the input active topology independent of the 
+/// transforms associated with either grid.
 template<typename GridType, typename InterruptT> inline
 typename VectorToScalarConverter<GridType>::Type::Ptr
 divergence(const GridType& grid, bool threaded, InterruptT* interrupt);
+
+template<typename GridType, typename MaskT, typename InterruptT> inline
+typename VectorToScalarConverter<GridType>::Type::Ptr
+divergence(const GridType& grid, const MaskT& mask, bool threaded, InterruptT* interrupt);
     
 template<typename GridType> inline
 typename VectorToScalarConverter<GridType>::Type::Ptr
@@ -115,13 +151,27 @@ divergence(const GridType& grid, bool threaded = true)
     return divergence<GridType, util::NullInterrupter>(grid, threaded, NULL);
 }
 
+template<typename GridType, typename MaskT> inline
+typename VectorToScalarConverter<GridType>::Type::Ptr
+divergence(const GridType& grid, const MaskT& mask, bool threaded = true)
+{
+    return divergence<GridType, MaskT, util::NullInterrupter>(grid, mask, threaded, NULL);
+}
+
 /// @brief Compute the gradient of the given scalar grid.
 /// @return a new vector-valued grid with the same numerical precision as the input grid
 ///     (for example, if the input grid is a DoubleGrid, the output grid will be a Vec3DGrid)
+/// When a mask grid is specified, the solution is calculated only in the intersection of
+/// the mask active topology and the input active topology independent of the 
+/// transforms associated with either grid.
 template<typename GridType, typename InterruptT> inline
 typename ScalarToVectorConverter<GridType>::Type::Ptr
 gradient(const GridType& grid, bool threaded, InterruptT* interrupt);
 
+template<typename GridType, typename MaskT, typename InterruptT> inline
+typename ScalarToVectorConverter<GridType>::Type::Ptr
+gradient(const GridType& grid, const MaskT& mask, bool threaded, InterruptT* interrupt);
+
 template<typename GridType> inline
 typename ScalarToVectorConverter<GridType>::Type::Ptr
 gradient(const GridType& grid, bool threaded = true)
@@ -130,25 +180,54 @@ gradient(const GridType& grid, bool threaded = true)
 }
 
 
+template<typename GridType, typename MaskT> inline
+typename ScalarToVectorConverter<GridType>::Type::Ptr
+gradient(const GridType& grid, const MaskT& mask, bool threaded = true)
+{
+    return gradient<GridType, MaskT, util::NullInterrupter>(grid, mask, threaded, NULL);
+}
+
+
 /// @brief Compute the Laplacian of the given scalar grid.
 /// @return a new scalar grid
+/// When a mask grid is specified, the solution is calculated only in the intersection of
+/// the mask active topology and the input active topology independent of the 
+/// transforms associated with either grid.
 template<typename GridType, typename InterruptT> inline
 typename GridType::Ptr
 laplacian(const GridType& grid, bool threaded, InterruptT* interrupt);
 
+template<typename GridType, typename MaskT, typename InterruptT> inline
+typename GridType::Ptr
+laplacian(const GridType& grid, const MaskT& mask, bool threaded, InterruptT* interrupt);
+
 template<typename GridType> inline
 typename GridType::Ptr
 laplacian(const GridType& grid, bool threaded = true)
 {
     return laplacian<GridType, util::NullInterrupter>(grid, threaded, NULL);
 }
+
+template<typename GridType, typename MaskT> inline
+typename GridType::Ptr
+laplacian(const GridType& grid, const MaskT mask, bool threaded = true)
+{
+    return laplacian<GridType, MaskT, util::NullInterrupter>(grid, mask, threaded, NULL);
+}
     
 /// @brief Compute the mean curvature of the given grid.
 /// @return a new grid
+/// When a mask grid is specified, the solution is calculated only in the intersection of
+/// the mask active topology and the input active topology independent of the 
+/// transforms associated with either grid.
 template<typename GridType, typename InterruptT> inline
 typename GridType::Ptr
 meanCurvature(const GridType& grid, bool threaded, InterruptT* interrupt);
 
+template<typename GridType, typename MaskT, typename InterruptT> inline
+typename GridType::Ptr
+meanCurvature(const GridType& grid, const MaskT& mask, bool threaded, InterruptT* interrupt);
+
 template<typename GridType> inline
 typename GridType::Ptr
 meanCurvature(const GridType& grid, bool threaded = true)    
@@ -156,12 +235,26 @@ meanCurvature(const GridType& grid, bool threaded = true)
     return meanCurvature<GridType, util::NullInterrupter>(grid, threaded, NULL);
 }
 
+template<typename GridType, typename MaskT> inline
+typename GridType::Ptr
+meanCurvature(const GridType& grid, const MaskT& mask, bool threaded = true)    
+{
+    return meanCurvature<GridType, MaskT, util::NullInterrupter>(grid, mask, threaded, NULL);
+}
+
 /// @brief Compute the magnitudes of the vectors of the given vector-valued grid.
 /// @return a new scalar-valued grid with the same numerical precision as the input grid
 ///     (for example, if the input grid is a Vec3DGrid, the output grid will be a DoubleGrid)
+/// When a mask grid is specified, the solution is calculated only in the intersection of
+/// the mask active topology and the input active topology independent of the 
+/// transforms associated with either grid.
 template<typename GridType, typename InterruptT> inline
 typename VectorToScalarConverter<GridType>::Type::Ptr
 magnitude(const GridType& grid, bool threaded, InterruptT* interrupt);
+
+template<typename GridType, typename MaskT, typename InterruptT> inline
+typename VectorToScalarConverter<GridType>::Type::Ptr
+magnitude(const GridType& grid, const MaskT& mask, bool threaded, InterruptT* interrupt);
     
 template<typename GridType> inline
 typename VectorToScalarConverter<GridType>::Type::Ptr
@@ -169,19 +262,40 @@ magnitude(const GridType& grid, bool threaded = true)
 {
     return magnitude<GridType, util::NullInterrupter>(grid, threaded, NULL);
 }
+
+template<typename GridType, typename MaskT> inline
+typename VectorToScalarConverter<GridType>::Type::Ptr
+magnitude(const GridType& grid, const MaskT& mask, bool threaded = true) 
+{
+    return magnitude<GridType, MaskT, util::NullInterrupter>(grid, mask, threaded, NULL);
+}
     
 /// @brief Normalize the vectors of the given vector-valued grid.
 /// @return a new vector-valued grid
+/// When a mask grid is specified, the solution is calculated only in the intersection of
+/// the mask active topology and the input active topology independent of the 
+/// transforms associated with either grid.
 template<typename GridType, typename InterruptT> inline
 typename GridType::Ptr
 normalize(const GridType& grid, bool threaded, InterruptT* interrupt);
 
+template<typename GridType, typename MaskT, typename InterruptT> inline
+typename GridType::Ptr
+normalize(const GridType& grid, const MaskT& mask, bool threaded, InterruptT* interrupt);
+
 template<typename GridType> inline
 typename GridType::Ptr
 normalize(const GridType& grid, bool threaded = true)
 {
     return normalize<GridType, util::NullInterrupter>(grid, threaded, NULL);
 }
+
+template<typename GridType, typename MaskT> inline
+typename GridType::Ptr
+normalize(const GridType& grid, const MaskT& mask, bool threaded = true)
+{
+    return normalize<GridType, MaskT, util::NullInterrupter>(grid, mask, threaded, NULL);
+}
     
 ////////////////////////////////////////
 
@@ -197,30 +311,40 @@ namespace {
 /// @note The current implementation assumes all the input
 /// values are represented by leaf voxels and not tiles. In the
 /// future we will expand this class to also handle tile values.
-template<typename InGridT, typename OutGridT, typename MapT,
-         typename OperatorT, typename InterruptT = util::NullInterrupter>
+template<typename InGridT, typename MaskGridType, typename OutGridT, typename MapT, typename OperatorT, typename InterruptT = util::NullInterrupter>
 class GridOperator
 {
 public:
     typedef typename OutGridT::TreeType           OutTreeT;
     typedef typename OutTreeT::LeafNodeType       OutLeafT;
     typedef typename tree::LeafManager<OutTreeT>  LeafManagerT;
-
-    GridOperator(const InGridT& grid, const MapT& map, InterruptT* interrupt = NULL):
-        mAcc(grid.getConstAccessor()), mMap(map), mInterrupt(interrupt)
+  
+    GridOperator(const InGridT& grid, const MaskGridType* mask, const MapT& map, InterruptT* interrupt = NULL):
+        mAcc(grid.getConstAccessor()), mMap(map), mInterrupt(interrupt), mMask(mask)
     {
     }
+  
     virtual ~GridOperator() {}
     typename OutGridT::Ptr process(bool threaded = true)
     {
         if (mInterrupt) mInterrupt->start("Processing grid");
+
         // Derive background value of the output grid
-        typename InGridT::TreeType tmp(mAcc.getTree()->background());
+        typename InGridT::TreeType tmp(mAcc.tree().background());
         typename OutGridT::ValueType backg = OperatorT::result(mMap, tmp, math::Coord(0));
+
         // output tree = topology copy of input tree!
-        typename OutTreeT::Ptr tree(new OutTreeT(*mAcc.getTree(), backg, TopologyCopy()));
+        typename OutTreeT::Ptr tree(new OutTreeT(mAcc.tree(), backg, TopologyCopy()));
+        
+        
         // create grid with output tree and unit transform
         typename OutGridT::Ptr result(new OutGridT(tree));
+        
+        // Modify the solution area if a mask was supplied.
+        if (mMask) {
+            result->topologyIntersection(*mMask); 
+        }
+
         // transform of output grid = transform of input grid
         result->setTransform(math::Transform::Ptr(new math::Transform( mMap.copy() )));
        
@@ -256,19 +380,20 @@ public:
 protected:
     
     typedef typename InGridT::ConstAccessor  AccessorT;
-    mutable AccessorT mAcc;
-    const MapT&       mMap;
-    InterruptT*       mInterrupt;
+    mutable AccessorT   mAcc;
+    const MapT&         mMap;
+    InterruptT*         mInterrupt;
+    const MaskGridType* mMask;
 }; // end of GridOperator class
 
-} //end of anonymous namespace
+} //end of anonymous namespaceo
 
 
 ////////////////////////////////////////
 
 
 /// @brief Compute the closest-point transform of a scalar grid.
-template<typename InGridT, typename InterruptT = util::NullInterrupter>
+template<typename InGridT, typename MaskGridType = BoolGrid, typename InterruptT = util::NullInterrupter>
 class Cpt
 {
 public:
@@ -276,12 +401,20 @@ public:
     typedef typename ScalarToVectorConverter<InGridT>::Type OutGridType;
 
     Cpt(const InGridType& grid, InterruptT* interrupt = NULL):
-        mInputGrid(grid), mInterrupt(interrupt)
+        mInputGrid(grid), mInterrupt(interrupt), mMask(NULL)
     {
     }
+
+
+    Cpt(const InGridType& grid, const MaskGridType& mask, InterruptT* interrupt = NULL):
+        mInputGrid(grid), mInterrupt(interrupt), mMask(&mask)
+    {
+    }
+
+
     typename OutGridType::Ptr process(bool threaded = true, bool useWorldTransform = true)
     {
-        Functor functor(mInputGrid, threaded, useWorldTransform, mInterrupt);
+        Functor functor(mInputGrid, mMask, threaded, useWorldTransform, mInterrupt);
         processTypedMap(mInputGrid.transform(), functor);
         return functor.mOutputGrid;
     }
@@ -306,16 +439,16 @@ private:
     };
     struct Functor
     {
-        Functor(const InGridType& grid, bool threaded, bool worldspace, InterruptT* interrupt):
-            mThreaded(threaded), mWorldSpace(worldspace), mInputGrid(grid), mInterrupt(interrupt){}
+        Functor(const InGridType& grid, const MaskGridType* mask, bool threaded, bool worldspace, InterruptT* interrupt):
+            mThreaded(threaded), mWorldSpace(worldspace), mInputGrid(grid), mInterrupt(interrupt), mMask(mask){}
         template<typename MapT>
         void operator()(const MapT& map)
         {
             if (mWorldSpace) {
-                GridOperator<InGridType, OutGridType, MapT, WsOpT, InterruptT> op(mInputGrid, map, mInterrupt);
+                GridOperator<InGridType, MaskGridType, OutGridType, MapT, WsOpT, InterruptT> op(mInputGrid, mMask, map, mInterrupt);
                 mOutputGrid = op.process(mThreaded); // cache the result
             } else {
-                GridOperator<InGridType, OutGridType, MapT, IsOpT, InterruptT> op(mInputGrid, map, mInterrupt);
+                GridOperator<InGridType, MaskGridType, OutGridType, MapT, IsOpT, InterruptT> op(mInputGrid, mMask, map, mInterrupt);
                 mOutputGrid = op.process(mThreaded); // cache the result
             }
         }
@@ -324,9 +457,11 @@ private:
         const InGridType&         mInputGrid;
         typename OutGridType::Ptr mOutputGrid;
         InterruptT*               mInterrupt;
+        const MaskGridType*       mMask;
     };
-    const InGridType& mInputGrid;
-    InterruptT*       mInterrupt;
+    const InGridType&   mInputGrid;
+    InterruptT*         mInterrupt;
+    const MaskGridType* mMask;
 }; // end of Cpt class
 
 
@@ -334,19 +469,24 @@ private:
 
 
 /// @brief Compute the curl of a scalar grid.
-template<typename GridT, typename InterruptT = util::NullInterrupter>
+template<typename GridT, typename MaskGridType = BoolGrid, typename InterruptT = util::NullInterrupter>
 class Curl
 {
 public:
     typedef GridT  InGridType;
     typedef GridT  OutGridType;
     Curl(const GridT& grid, InterruptT* interrupt = NULL):
-        mInputGrid(grid), mInterrupt(interrupt)
+        mInputGrid(grid), mInterrupt(interrupt), mMask(NULL)
     {
     }
+    Curl(const GridT& grid, const MaskGridType& mask, InterruptT* interrupt = NULL):
+        mInputGrid(grid), mInterrupt(interrupt), mMask(&mask)
+    {
+    }
+
     typename GridT::Ptr process(bool threaded = true)
     {
-        Functor functor(mInputGrid, threaded, mInterrupt);
+        Functor functor(mInputGrid, mMask, threaded, mInterrupt);
         processTypedMap(mInputGrid.transform(), functor);
         return functor.mOutputGrid;
     }
@@ -354,23 +494,26 @@ public:
 private:
     struct Functor
     {
-        Functor(const GridT& grid, bool threaded, InterruptT* interrupt):
-            mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt){}
+        Functor(const GridT& grid, const MaskGridType* mask, 
+                bool threaded, InterruptT* interrupt):
+            mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt), mMask(mask){}
         template<typename MapT>
         void operator()(const MapT& map)
         {
             typedef math::Curl<MapT, math::CD_2ND> OpT;
-            GridOperator<GridT, GridT, MapT, OpT, InterruptT> op(mInputGrid, map, mInterrupt);
+            GridOperator<GridT, MaskGridType, GridT, MapT, OpT, InterruptT> op(mInputGrid, mMask, map, mInterrupt);
             mOutputGrid = op.process(mThreaded); // cache the result
         }
         const bool           mThreaded;
         const GridT&         mInputGrid;
         typename GridT::Ptr  mOutputGrid;
         InterruptT*          mInterrupt;
+        const MaskGridType*  mMask;
     }; // Private Functor
 
-    const GridT& mInputGrid;
-    InterruptT*  mInterrupt;
+    const GridT&         mInputGrid;
+    InterruptT*          mInterrupt;
+    const MaskGridType*  mMask;
 }; // end of Curl class
 
 
@@ -378,7 +521,7 @@ private:
 
 
 /// @brief Computes the Divergence of a scalar grid
-template<typename InGridT, typename InterruptT = util::NullInterrupter>
+template<typename InGridT, typename MaskGridType = BoolGrid, typename InterruptT = util::NullInterrupter>
 class Divergence
 {
 public:
@@ -386,18 +529,23 @@ public:
     typedef typename VectorToScalarConverter<InGridT>::Type OutGridType;
 
     Divergence(const InGridT& grid, InterruptT* interrupt = NULL):
-        mInputGrid(grid), mInterrupt(interrupt)
+        mInputGrid(grid), mInterrupt(interrupt), mMask(NULL)
+    {
+    }
+
+    Divergence(const InGridT& grid, const MaskGridType& mask, InterruptT* interrupt = NULL):
+        mInputGrid(grid), mInterrupt(interrupt), mMask(&mask)
     {
     }
     typename OutGridType::Ptr process(bool threaded = true)
     {
         if( mInputGrid.getGridClass() == GRID_STAGGERED ) {
-            Functor<math::FD_1ST> functor(mInputGrid, threaded, mInterrupt);
+            Functor<math::FD_1ST> functor(mInputGrid, mMask, threaded, mInterrupt);
             processTypedMap(mInputGrid.transform(), functor);
             return functor.mOutputGrid;
         }
         else {
-            Functor<math::CD_2ND> functor(mInputGrid, threaded, mInterrupt);
+            Functor<math::CD_2ND> functor(mInputGrid, mMask, threaded, mInterrupt);
             processTypedMap(mInputGrid.transform(), functor);
             return functor.mOutputGrid;
         }
@@ -407,23 +555,25 @@ protected:
     template<math::DScheme DiffScheme>
     struct Functor
     {
-        Functor(const InGridT& grid, bool threaded, InterruptT* interrupt):
-            mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt) {}
+        Functor(const InGridT& grid, const MaskGridType* mask, bool threaded, InterruptT* interrupt):
+            mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt), mMask(mask) {}
         template<typename MapT>
         void operator()(const MapT& map)
         {
             typedef math::Divergence<MapT, DiffScheme> OpT;
-            GridOperator<InGridType, OutGridType, MapT, OpT, InterruptT> op(mInputGrid, map, mInterrupt);
+            GridOperator<InGridType, MaskGridType, OutGridType, MapT, OpT, InterruptT> op(mInputGrid, mMask, map, mInterrupt);
             mOutputGrid = op.process(mThreaded); // cache the result
         }
         const bool                 mThreaded;
         const InGridType&          mInputGrid;
         typename OutGridType::Ptr  mOutputGrid;
         InterruptT*                mInterrupt;
+        const MaskGridType*        mMask;
     }; // Private Functor
 
-    const InGridType& mInputGrid;
-    InterruptT*       mInterrupt;
+    const InGridType&    mInputGrid;
+    InterruptT*          mInterrupt;
+    const MaskGridType*  mMask;
 }; // end of Divergence class
 
 
@@ -431,7 +581,7 @@ protected:
 
 
 /// @brief Computes the Gradient of a scalar grid
-template<typename InGridT, typename InterruptT = util::NullInterrupter>
+template<typename InGridT, typename MaskGridType = BoolGrid, typename InterruptT = util::NullInterrupter>
 class Gradient
 {
 public:
@@ -439,12 +589,16 @@ public:
     typedef typename ScalarToVectorConverter<InGridT>::Type OutGridType;
 
     Gradient(const InGridT& grid, InterruptT* interrupt = NULL):
-        mInputGrid(grid), mInterrupt(interrupt)
+        mInputGrid(grid), mInterrupt(interrupt), mMask(NULL)
+    {
+    }
+    Gradient(const InGridT& grid, const MaskGridType& mask, InterruptT* interrupt = NULL):
+        mInputGrid(grid), mInterrupt(interrupt), mMask(&mask)
     {
     }
     typename OutGridType::Ptr process(bool threaded = true)
     {
-        Functor functor(mInputGrid, threaded, mInterrupt);
+        Functor functor(mInputGrid, mMask, threaded, mInterrupt);
         processTypedMap(mInputGrid.transform(), functor);
         return functor.mOutputGrid;
     }
@@ -452,23 +606,25 @@ public:
 protected:
     struct Functor
     {
-        Functor(const InGridT& grid, bool threaded, InterruptT* interrupt):
-            mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt) {}
+        Functor(const InGridT& grid, const MaskGridType* mask, bool threaded, InterruptT* interrupt):
+            mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt), mMask(mask) {}
         template<typename MapT>
         void operator()(const MapT& map)
         {
             typedef math::Gradient<MapT, math::CD_2ND> OpT;
-            GridOperator<InGridType, OutGridType, MapT, OpT, InterruptT> op(mInputGrid, map, mInterrupt);
+            GridOperator<InGridType, MaskGridType, OutGridType, MapT, OpT, InterruptT> op(mInputGrid, mMask, map, mInterrupt);
             mOutputGrid = op.process(mThreaded); // cache the result
         }
         const bool                 mThreaded;
         const InGridT&             mInputGrid;
         typename OutGridType::Ptr  mOutputGrid;
         InterruptT*                mInterrupt;
+        const MaskGridType*        mMask;
     }; // Private Functor
 
-    const InGridT& mInputGrid;
-    InterruptT*    mInterrupt;
+    const InGridT&       mInputGrid;
+    InterruptT*          mInterrupt;
+    const MaskGridType*  mMask;
 }; // end of Gradient class
 
 
@@ -476,19 +632,24 @@ protected:
 
 
 /// @brief Computes the Laplacian of a scalar grid
-template<typename GridT, typename InterruptT = util::NullInterrupter>
+template<typename GridT, typename MaskGridType = BoolGrid, typename InterruptT = util::NullInterrupter>
 class Laplacian
 {
 public:
     typedef GridT  InGridType;
     typedef GridT  OutGridType;
     Laplacian(const GridT& grid, InterruptT* interrupt = NULL):
-        mInputGrid(grid), mInterrupt(interrupt)
+        mInputGrid(grid), mInterrupt(interrupt), mMask(NULL)
+    {
+    }
+    
+    Laplacian(const GridT& grid, const MaskGridType& mask, InterruptT* interrupt = NULL):
+        mInputGrid(grid), mInterrupt(interrupt), mMask(&mask)
     {
     }
     typename GridT::Ptr process(bool threaded = true)
     {
-        Functor functor(mInputGrid, threaded, mInterrupt);
+        Functor functor(mInputGrid, mMask, threaded, mInterrupt);
         processTypedMap(mInputGrid.transform(), functor);
         return functor.mOutputGrid;
     }
@@ -496,42 +657,49 @@ public:
 protected:
     struct Functor
     {
-        Functor(const GridT& grid, bool threaded, InterruptT* interrupt):
-            mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt) {}
+        Functor(const GridT& grid, const MaskGridType* mask, bool threaded, InterruptT* interrupt):
+            mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt), mMask(mask) {}
         template<typename MapT>
         void operator()(const MapT& map)
         {
             typedef math::Laplacian<MapT, math::CD_SECOND> OpT;
-            GridOperator<GridT, GridT, MapT, OpT, InterruptT> op(mInputGrid, map);
+            GridOperator<GridT, MaskGridType, GridT, MapT, OpT, InterruptT> op(mInputGrid, mMask, map);
             mOutputGrid = op.process(mThreaded); // cache the result
         }
         const bool           mThreaded;
         const GridT&         mInputGrid;
         typename GridT::Ptr  mOutputGrid;
         InterruptT*          mInterrupt;
+        const MaskGridType*  mMask;
     }; // Private Functor
 
-    const GridT& mInputGrid;
-    InterruptT*  mInterrupt;
+    const GridT&        mInputGrid;
+    InterruptT*         mInterrupt;
+    const MaskGridType* mMask;
 }; // end of Laplacian class
 
 
 ////////////////////////////////////////
 
 
-template<typename GridT, typename InterruptT = util::NullInterrupter>
+template<typename GridT, typename MaskGridType = BoolGrid, typename InterruptT = util::NullInterrupter>
 class MeanCurvature
 {
 public:
     typedef GridT  InGridType;
     typedef GridT  OutGridType;
     MeanCurvature(const GridT& grid, InterruptT* interrupt = NULL):
-        mInputGrid(grid), mInterrupt(interrupt)
+        mInputGrid(grid), mInterrupt(interrupt), mMask(NULL)
     {
     }
+    MeanCurvature(const GridT& grid, const MaskGridType& mask, InterruptT* interrupt = NULL):
+        mInputGrid(grid), mInterrupt(interrupt), mMask(&mask)
+    {
+    }
+
     typename GridT::Ptr process(bool threaded = true)
     {
-        Functor functor(mInputGrid, threaded, mInterrupt);
+        Functor functor(mInputGrid, mMask, threaded, mInterrupt);
         processTypedMap(mInputGrid.transform(), functor);
         return functor.mOutputGrid;
     }
@@ -539,42 +707,48 @@ public:
 protected:
     struct Functor
     {
-        Functor(const GridT& grid, bool threaded, InterruptT* interrupt):
-            mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt) {}
+        Functor(const GridT& grid, const MaskGridType* mask, bool threaded, InterruptT* interrupt):
+            mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt), mMask(mask) {}
         template<typename MapT>
         void operator()(const MapT& map)
         {
             typedef math::MeanCurvature<MapT, math::CD_SECOND, math::CD_2ND> OpT;
-            GridOperator<GridT, GridT, MapT, OpT, InterruptT> op(mInputGrid, map);
+            GridOperator<GridT, MaskGridType, GridT, MapT, OpT, InterruptT> op(mInputGrid, mMask, map);
             mOutputGrid = op.process(mThreaded); // cache the result
         }
         const bool           mThreaded;
         const GridT&         mInputGrid;
         typename GridT::Ptr  mOutputGrid;
         InterruptT*          mInterrupt;
+        const MaskGridType*  mMask; 
     }; // Private Functor
 
-    const GridT& mInputGrid;
-    InterruptT*  mInterrupt;
+    const GridT&        mInputGrid;
+    InterruptT*         mInterrupt;
+    const MaskGridType* mMask;
 }; // end of MeanCurvature class
 
 
 ////////////////////////////////////////
 
 
-template<typename InGridT, typename InterruptT = util::NullInterrupter>
+template<typename InGridT, typename MaskGridType = BoolGrid, typename InterruptT = util::NullInterrupter>
 class Magnitude
 {
 public:
     typedef InGridT                                         InGridType;
     typedef typename VectorToScalarConverter<InGridT>::Type OutGridType;
     Magnitude(const InGridType& grid, InterruptT* interrupt = NULL):
-        mInputGrid(grid), mInterrupt(interrupt)
+        mInputGrid(grid), mInterrupt(interrupt), mMask(NULL)
+    {
+    }
+    Magnitude(const InGridType& grid, const MaskGridType& mask, InterruptT* interrupt = NULL):
+        mInputGrid(grid), mInterrupt(interrupt), mMask(&mask)
     {
     }
     typename OutGridType::Ptr process(bool threaded = true)
     {
-        Functor functor(mInputGrid, threaded, mInterrupt);
+        Functor functor(mInputGrid, mMask, threaded, mInterrupt);
         processTypedMap(mInputGrid.transform(), functor);
         return functor.mOutputGrid;
     }
@@ -588,41 +762,48 @@ protected:
     };
     struct Functor
     {
-        Functor(const InGridT& grid, bool threaded, InterruptT* interrupt):
-            mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt) {}
+        Functor(const InGridT& grid, const MaskGridType* mask, bool threaded, InterruptT* interrupt):
+            mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt), mMask(mask) {}
         template<typename MapT>
         void operator()(const MapT& map)
         {
-            GridOperator<InGridType, OutGridType, MapT, OpT, InterruptT> op(mInputGrid, map);
+            GridOperator<InGridType, MaskGridType, OutGridType, MapT, OpT, InterruptT> op(mInputGrid, mMask, map);
             mOutputGrid = op.process(mThreaded); // cache the result
         }
         const bool                 mThreaded;
         const InGridType&          mInputGrid;
         typename OutGridType::Ptr  mOutputGrid;
         InterruptT*                mInterrupt;
+        const MaskGridType*        mMask;
     }; // Private Functor
 
-    const InGridType& mInputGrid;
-    InterruptT*       mInterrupt;
+    const InGridType&    mInputGrid;
+    InterruptT*          mInterrupt;
+    const MaskGridType*  mMask;
 }; // end of Magnitude class
 
 
 ////////////////////////////////////////
 
 
-template<typename GridT, typename InterruptT = util::NullInterrupter>
+template<typename GridT, typename MaskGridType = BoolGrid, typename InterruptT = util::NullInterrupter>
 class Normalize
 {
 public:
     typedef GridT  InGridType;
     typedef GridT  OutGridType;
     Normalize(const GridT& grid, InterruptT* interrupt = NULL):
-        mInputGrid(grid), mInterrupt(interrupt)
+        mInputGrid(grid), mInterrupt(interrupt), mMask(NULL)
     {
     }
+    Normalize(const GridT& grid, const MaskGridType& mask, InterruptT* interrupt = NULL):
+        mInputGrid(grid), mInterrupt(interrupt), mMask(&mask)
+    {
+    }
+  
     typename GridT::Ptr process(bool threaded = true)
     {
-        Functor functor(mInputGrid, threaded, mInterrupt);
+        Functor functor(mInputGrid, mMask, threaded, mInterrupt);
         processTypedMap(mInputGrid.transform(), functor);
         return functor.mOutputGrid;
     }
@@ -641,22 +822,24 @@ protected:
     };
     struct Functor
     {
-        Functor(const GridT& grid, bool threaded, InterruptT* interrupt):
-            mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt) {}
+        Functor(const GridT& grid, const MaskGridType* mask, bool threaded, InterruptT* interrupt):
+            mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt), mMask(mask) {}
         template<typename MapT>
         void operator()(const MapT& map)
         {
-            GridOperator<GridT, GridT, MapT, OpT, InterruptT> op(mInputGrid, map);
+            GridOperator<GridT, MaskGridType, GridT, MapT, OpT, InterruptT> op(mInputGrid, mMask,map);
             mOutputGrid = op.process(mThreaded); // cache the result
         }
         const bool           mThreaded;
         const GridT&         mInputGrid;
         typename GridT::Ptr  mOutputGrid;
         InterruptT*          mInterrupt;
+        const MaskGridType*  mMask;
     }; // Private Functor
 
-    const GridT& mInputGrid;
-    InterruptT*  mInterrupt;
+    const GridT&        mInputGrid;
+    InterruptT*         mInterrupt;
+    const MaskGridType* mMask;
 }; // end of Normalize class
 
 
@@ -667,7 +850,15 @@ template<typename GridType, typename InterruptT> inline
 typename ScalarToVectorConverter<GridType>::Type::Ptr
 cpt(const GridType& grid, bool threaded, InterruptT* interrupt)
 {
-    Cpt<GridType, InterruptT> op(grid, interrupt);
+    Cpt<GridType, BoolGrid, InterruptT> op(grid, interrupt);
+    return op.process(threaded);
+}
+
+template<typename GridType, typename MaskT, typename InterruptT> inline
+typename ScalarToVectorConverter<GridType>::Type::Ptr
+cpt(const GridType& grid, const MaskT& mask, bool threaded, InterruptT* interrupt)
+{
+    Cpt<GridType, MaskT, InterruptT> op(grid, mask, interrupt);
     return op.process(threaded);
 }
 
@@ -675,7 +866,16 @@ template<typename GridType, typename InterruptT> inline
 typename GridType::Ptr
 curl(const GridType& grid, bool threaded, InterruptT* interrupt)
 {
-    Curl<GridType, InterruptT> op(grid, interrupt);
+    Curl<GridType, BoolGrid, InterruptT> op(grid, interrupt);
+    return op.process(threaded);
+}
+
+
+template<typename GridType, typename MaskT, typename InterruptT> inline
+typename GridType::Ptr
+curl(const GridType& grid, const MaskT& mask, bool threaded, InterruptT* interrupt)
+{
+    Curl<GridType, MaskT, InterruptT> op(grid, mask, interrupt);
     return op.process(threaded);
 }
 
@@ -683,7 +883,15 @@ template<typename GridType, typename InterruptT> inline
 typename VectorToScalarConverter<GridType>::Type::Ptr
 divergence(const GridType& grid, bool threaded, InterruptT* interrupt)
 {
-    Divergence<GridType, InterruptT> op(grid, interrupt);
+    Divergence<GridType, BoolGrid, InterruptT> op(grid, interrupt);
+    return op.process(threaded);
+}
+
+template<typename GridType, typename MaskT, typename InterruptT> inline
+typename VectorToScalarConverter<GridType>::Type::Ptr
+divergence(const GridType& grid, const MaskT& mask, bool threaded, InterruptT* interrupt)
+{
+    Divergence<GridType, MaskT, InterruptT> op(grid, mask, interrupt);
     return op.process(threaded);
 }
 
@@ -691,7 +899,15 @@ template<typename GridType, typename InterruptT> inline
 typename ScalarToVectorConverter<GridType>::Type::Ptr
 gradient(const GridType& grid, bool threaded, InterruptT* interrupt)
 {
-    Gradient<GridType, InterruptT> op(grid, interrupt);
+    Gradient<GridType, BoolGrid, InterruptT> op(grid, interrupt);
+    return op.process(threaded);
+}
+
+template<typename GridType, typename MaskT, typename InterruptT> inline
+typename ScalarToVectorConverter<GridType>::Type::Ptr
+gradient(const GridType& grid, const MaskT& mask, bool threaded, InterruptT* interrupt)
+{
+    Gradient<GridType, MaskT, InterruptT> op(grid, mask, interrupt);
     return op.process(threaded);
 }
 
@@ -699,7 +915,15 @@ template<typename GridType, typename InterruptT> inline
 typename GridType::Ptr
 laplacian(const GridType& grid, bool threaded, InterruptT* interrupt)
 {
-    Laplacian<GridType, InterruptT> op(grid, interrupt);
+    Laplacian<GridType, BoolGrid, InterruptT> op(grid, interrupt);
+    return op.process(threaded);
+}
+
+template<typename GridType, typename MaskT, typename InterruptT> inline
+typename GridType::Ptr
+laplacian(const GridType& grid, const MaskT& mask, bool threaded, InterruptT* interrupt)
+{
+    Laplacian<GridType, MaskT, InterruptT> op(grid, mask, interrupt);
     return op.process(threaded);
 }
 
@@ -707,7 +931,15 @@ template<typename GridType, typename InterruptT> inline
 typename GridType::Ptr
 meanCurvature(const GridType& grid, bool threaded, InterruptT* interrupt)
 {
-    MeanCurvature<GridType, InterruptT> op(grid, interrupt);
+    MeanCurvature<GridType, BoolGrid, InterruptT> op(grid, interrupt);
+    return op.process(threaded);
+}
+
+template<typename GridType, typename MaskT, typename InterruptT> inline
+typename GridType::Ptr
+meanCurvature(const GridType& grid, const MaskT& mask, bool threaded, InterruptT* interrupt)
+{
+    MeanCurvature<GridType, MaskT, InterruptT> op(grid, mask, interrupt);
     return op.process(threaded);
 }
 
@@ -715,7 +947,15 @@ template<typename GridType, typename InterruptT> inline
 typename VectorToScalarConverter<GridType>::Type::Ptr
 magnitude(const GridType& grid, bool threaded, InterruptT* interrupt)
 {
-    Magnitude<GridType, InterruptT> op(grid, interrupt);
+    Magnitude<GridType, BoolGrid, InterruptT> op(grid, interrupt);
+    return op.process(threaded);
+}
+
+template<typename GridType, typename MaskT, typename InterruptT> inline
+typename VectorToScalarConverter<GridType>::Type::Ptr
+magnitude(const GridType& grid, const MaskT& mask, bool threaded, InterruptT* interrupt)
+{
+    Magnitude<GridType, MaskT, InterruptT> op(grid, mask, interrupt);
     return op.process(threaded);
 }
 
@@ -723,7 +963,15 @@ template<typename GridType, typename InterruptT> inline
 typename GridType::Ptr
 normalize(const GridType& grid, bool threaded, InterruptT* interrupt)
 {
-    Normalize<GridType, InterruptT> op(grid, interrupt);
+    Normalize<GridType, BoolGrid, InterruptT> op(grid, interrupt);
+    return op.process(threaded);
+}
+
+template<typename GridType, typename MaskT, typename InterruptT> inline
+typename GridType::Ptr
+normalize(const GridType& grid, const MaskT& mask, bool threaded, InterruptT* interrupt)
+{
+    Normalize<GridType, MaskT, InterruptT> op(grid, mask, interrupt);
     return op.process(threaded);
 }
 
