@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -39,17 +39,19 @@
 #ifndef OPENVDB_TOOLS_LEVELSETSPHERE_HAS_BEEN_INCLUDED
 #define OPENVDB_TOOLS_LEVELSETSPHERE_HAS_BEEN_INCLUDED
 
-#include <openvdb/openvdb.h>
+#include <openvdb/Grid.h>
+#include <openvdb/Types.h>
+#include <openvdb/math/Math.h>
 #include <openvdb/util/NullInterrupter.h>
 #include <boost/utility.hpp>
 #include <boost/type_traits/is_floating_point.hpp>
 
+
 namespace openvdb {
 OPENVDB_USE_VERSION_NAMESPACE
 namespace OPENVDB_VERSION_NAME {
 namespace tools {
 
-
 /// @brief Return a grid of type @c GridType containing a narrow-band level set
 /// representation of a sphere.
 ///
@@ -87,7 +89,8 @@ createLevelSetSphere(float radius, const openvdb::Vec3f& center, float voxelSize
 {
     return createLevelSetSphere<GridType, util::NullInterrupter>(radius,center,voxelSize,halfWidth);
 }
-    
+
+
 ////////////////////////////////////////
 
 
@@ -133,9 +136,7 @@ public:
         return mGrid;
     }
 
-
 private:
-
     void rasterSphere(ValueT dx, ValueT w)
     {
         if (!(dx>0.0f)) OPENVDB_THROW(ValueError, "voxel size must be positive");
@@ -172,10 +173,11 @@ private:
                     /// Distance in voxel units to sphere
                     const float v = math::Sqrt(x2y2 + math::Pow2(k-c[2]))-r0,
                         d = math::Abs(v);
-                    if ( d < w )// inside narrow band
+                    if ( d < w ){ // inside narrow band
                         accessor.setValue(ijk, dx*v);// distance in world units
-                    else// outside narrow band
+                    } else {// outside narrow band
                         m += math::Floor(d-w);// leapfrog
+                    }    
                 }//end leapfrog over k
             }//end loop over j
         }//end loop over i
@@ -194,6 +196,7 @@ private:
 
 ////////////////////////////////////////
 
+
 template<typename GridType, typename InterruptT>
 typename GridType::Ptr
 createLevelSetSphere(float radius, const openvdb::Vec3f& center, float voxelSize,
@@ -207,38 +210,12 @@ createLevelSetSphere(float radius, const openvdb::Vec3f& center, float voxelSize
     return factory.getLevelSet(ValueT(voxelSize), ValueT(halfWidth));
 }
 
-
-////////////////////////////////////////
-
-
-/// @brief @return a FloatGrid containing a narrow-band SDF of a sphere
-///
-/// @param radius    Radius of the sphere in world units
-/// @param center    Center of the sphere in world units
-/// @param voxelSize Size of voxels in world units
-/// @param halfWidth Half-width of narrow-band in voxel units
-/// @param interrupt Pointer to optional interrupter. Use template
-/// argument util::NullInterrupter if no interruption is desired.
-///
-/// @note the leapfrog algorithm employed in this method is best
-/// suited for a single large sphere. For multiple small spheres consider
-/// using the faster algorithm in ParticlesToLevelSet.h
-///
-/// @deprecated Use createSphereSDF() or LevelSetSphere instead.
-template<typename InterruptT>
-OPENVDB_DEPRECATED FloatGrid::Ptr makeSphere(float radius, const openvdb::Vec3f &center,
-    float voxelSize, float halfWidth, InterruptT* interrupt)
-{
-    LevelSetSphere<FloatGrid, InterruptT> s(radius, center, &interrupt);
-    return s.getLevelSet(voxelSize, halfWidth);
-}
-
 } // namespace tools
 } // namespace OPENVDB_VERSION_NAME
 } // namespace openvdb
 
 #endif // OPENVDB_TOOLS_LEVELSETSPHERE_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
