@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -41,7 +41,7 @@
 #include <openvdb/Types.h>                 // Vec3 types and version number
 #include <openvdb/Grid.h>                  // grid
 #include <openvdb/util/NullInterrupter.h>
-#include "GridTransformer.h"               // sampling
+#include "Interpolation.h"                 // sampling
 
 #include <boost/static_assert.hpp>
 #include <tbb/blocked_range.h>             // threading
@@ -519,6 +519,6 @@ private:
 
 #endif // OPENVDB_TOOLS_POINT_ADVECT_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
