@@ -72,13 +72,13 @@ public:
 
     /// Return the name of this tree's type.
     virtual const Name& type() const = 0;
+    
     /// Return the name of the type of a voxel's value (e.g., "float" or "vec3d").
     virtual Name valueType() const = 0;
 
     /// Return a pointer to a deep copy of this tree
     virtual TreeBase::Ptr copy() const = 0;
 
-
     //
     // Tree methods
     //
@@ -86,10 +86,15 @@ public:
     /// @note Query the metadata object for the value's type.
     virtual Metadata::Ptr getBackgroundValue() const { return Metadata::Ptr(); }
 
-    /// @brief Return in @a bbox the axis-aligned bounding box of all leaf nodes.
+    /// @brief Return in @a bbox the axis-aligned bounding box of all
+    /// leaf nodes and active tiles.
+    /// @details This is faster then calling evalActiveVoxelBoundingBox, 
+    /// which visits the individual active voxels, and hence
+    /// evalLeafBoundingBox produces a less tight, i.e. approximate, bbox.
     /// @return @c false if the bounding box is empty (in which case
     /// the bbox is set to its default value).
     virtual bool evalLeafBoundingBox(CoordBBox& bbox) const = 0;
+    
     /// @brief Return in @a dim the dimensions of the axis-aligned bounding box
     /// of all leaf nodes.
     /// @return @c false if the bounding box is empty.
@@ -97,10 +102,13 @@ public:
 
     /// @brief Return in @a bbox the axis-aligned bounding box of all
     /// active voxels and tiles.
-    /// This is a tighter bounding box than the leaf node bounding box.
+    /// @details This method produces a more accurate, i.e. tighter,
+    /// bounding box than evalLeafBoundingBox which is approximate but
+    /// faster.
     /// @return @c false if the bounding box is empty (in which case
     /// the bbox is set to its default value).
     virtual bool evalActiveVoxelBoundingBox(CoordBBox& bbox) const = 0;
+    
     /// @brief Return in @a dim the dimensions of the axis-aligned bounding box of all
     /// active voxels.  This is a tighter bounding box than the leaf node bounding box.
     /// @return @c false if the bounding box is empty.
@@ -248,7 +256,7 @@ public:
     /// Return the name of this type of tree.
     static const Name& treeType();
     /// Return the name of this type of tree.
-    virtual const Name& type() const { return treeType(); }
+    virtual const Name& type() const { return this->treeType(); }
 
     bool operator==(const Tree&) const { OPENVDB_THROW(NotImplementedError, ""); }
     bool operator!=(const Tree&) const { OPENVDB_THROW(NotImplementedError, ""); }
@@ -267,7 +275,7 @@ public:
     // Tree methods
     //
     /// @brief Return @c true if the given tree has the same node and active value
-    /// topology as this tree (but possibly a different @c ValueType).
+    /// topology as this tree, whether or not it has the same @c ValueType.
     template<typename OtherRootNodeType>
     bool hasSameTopology(const Tree<OtherRootNodeType>& other) const;
 
@@ -321,6 +329,10 @@ public:
     /// Return the number of inactive voxels within the bounding box of all active voxels.
     virtual Index64 inactiveVoxelCount() const;
 
+    /// Return the total number of active tiles.
+    /// @note This method is not virtual so as to not change the ABI.
+    Index64 activeTileCount() const { return mRoot.onTileCount(); }
+    
     /// Return the minimum and maximum active values in this tree.
     void evalMinMax(ValueType &min, ValueType &max) const;
 
@@ -337,39 +349,70 @@ public:
     template<typename AccessT> const ValueType& getValue(const Coord& xyz, AccessT&) const;
 
     /// @brief Return the tree depth (0 = root) at which the value of voxel (x, y, z) resides.
-    ///
-    /// If (x, y, z) isn't explicitly represented in the tree (i.e., it is implicitly
-    /// a background voxel), return -1.
+    /// @details If (x, y, z) isn't explicitly represented in the tree (i.e., it is
+    /// implicitly a background voxel), return -1.
     int getValueDepth(const Coord& xyz) const;
 
-    /// Set the value of the voxel at the given coordinates and mark the voxel as active.
-    void setValue(const Coord& xyz, const ValueType& value);
-    /// Set the value of the voxel at the given coordinates but preserve it active state.
+    /// Set the active state of the voxel at the given coordinates but don't change its value.
+    void setActiveState(const Coord& xyz, bool on);
+    /// Set the value of the voxel at the given coordinates but don't change its active state.
     void setValueOnly(const Coord& xyz, const ValueType& value);
-    /// @brief Set the value of the voxel at the given coordinates, mark the voxel as active,
-    /// and update the given accessor's node cache.
-    template<typename AccessT> void setValue(const Coord& xyz, const ValueType& value, AccessT&);
-    /// Mark the voxel at the given coordinates as active, but don't change its value.
+    /// Mark the voxel at the given coordinates as active but don't change its value.
     void setValueOn(const Coord& xyz);
     /// Set the value of the voxel at the given coordinates and mark the voxel as active.
     void setValueOn(const Coord& xyz, const ValueType& value);
-    /// @brief Set the value of the voxel at the given coordinates to the minimum
-    /// of its current value and the given value, and mark the voxel as active.
-    void setValueOnMin(const Coord& xyz, const ValueType& value);
-    /// @brief Set the value of the voxel at the given coordinates to the maximum
-    /// of its current value and the given value, and mark the voxel as active.
-    void setValueOnMax(const Coord& xyz, const ValueType& value);
-    /// @brief Set the value of the voxel at the given coordinates to the sum
-    /// of its current value and the given value, and mark the voxel as active.
-    void setValueOnSum(const Coord& xyz, const ValueType& value);
-
-    /// Mark the voxel at the given coordinates as inactive, but don't change its value.
+    /// Set the value of the voxel at the given coordinates and mark the voxel as active.
+    void setValue(const Coord& xyz, const ValueType& value);
+    /// @brief Set the value of the voxel at the given coordinates, mark the voxel as active,
+    /// and update the given accessor's node cache.
+    template<typename AccessT> void setValue(const Coord& xyz, const ValueType& value, AccessT&);
+    /// Mark the voxel at the given coordinates as inactive but don't change its value.
     void setValueOff(const Coord& xyz);
-    /// Change the value of the voxel at the given coordinates and mark the voxel as inactive.
+    /// Set the value of the voxel at the given coordinates and mark the voxel as inactive.
     void setValueOff(const Coord& xyz, const ValueType& value);
 
-    /// Set the active state of the voxel at the given coordinates, but don't change its value.
-    void setActiveState(const Coord& xyz, bool on);
+    /// @brief Apply a functor to the value of the voxel at the given coordinates
+    /// and mark the voxel as active.
+    /// @details Provided that the functor can be inlined, this is typically
+    /// significantly faster than calling getValue() followed by setValueOn().
+    /// @param xyz  the coordinates of a voxel whose value is to be modified
+    /// @param op   a functor of the form <tt>void op(ValueType&) const</tt> that modifies
+    ///             its argument in place
+    /// @par Example:
+    /// @code
+    /// Coord xyz(1, 0, -2);
+    /// // Multiply the value of a voxel by a constant and mark the voxel as active.
+    /// floatTree.modifyValue(xyz, [](float& f) { f *= 0.25; }); // C++11
+    /// // Set the value of a voxel to the maximum of its current value and 0.25,
+    /// // and mark the voxel as active.
+    /// floatTree.modifyValue(xyz, [](float& f) { f = std::max(f, 0.25f); }); // C++11
+    /// @endcode
+    /// @note The functor is not guaranteed to be called only once.
+    /// @see tools::foreach()
+    template<typename ModifyOp>
+    void modifyValue(const Coord& xyz, const ModifyOp& op);
+
+    /// @brief Apply a functor to the voxel at the given coordinates.
+    /// @details Provided that the functor can be inlined, this is typically
+    /// significantly faster than calling getValue() followed by setValue().
+    /// @param xyz  the coordinates of a voxel to be modified
+    /// @param op   a functor of the form <tt>void op(ValueType&, bool&) const</tt> that
+    ///             modifies its arguments, a voxel's value and active state, in place
+    /// @par Example:
+    /// @code
+    /// Coord xyz(1, 0, -2);
+    /// // Multiply the value of a voxel by a constant and mark the voxel as inactive.
+    /// floatTree.modifyValueAndActiveState(xyz,
+    ///     [](float& f, bool& b) { f *= 0.25; b = false; }); // C++11
+    /// // Set the value of a voxel to the maximum of its current value and 0.25,
+    /// // but don't change the voxel's active state.
+    /// floatTree.modifyValueAndActiveState(xyz,
+    ///     [](float& f, bool&) { f = std::max(f, 0.25f); }); // C++11
+    /// @endcode
+    /// @note The functor is not guaranteed to be called only once.
+    /// @see tools::foreach()
+    template<typename ModifyOp>
+    void modifyValueAndActiveState(const Coord& xyz, const ModifyOp& op);
 
     /// @brief Get the value of the voxel at the given coordinates.
     /// @return @c true if the value is active.
@@ -414,13 +457,12 @@ public:
     /// background tiles any nodes whose values are all inactive.
     void pruneInactive();
 
-    /// @brief Prune any descendants whose values are all inactive and replace them
-    /// with inactive tiles having a values equal to the first value
-    /// encountered in the (inactive) child.
-    ///
-    /// @note This method is faster then tolerance based prune and
+    /// @brief Reduce the memory footprint of this tree by replacing nodes
+    /// whose values are all inactive with inactive tiles having a value equal to
+    /// the first value encountered in the (inactive) child.
+    /// @details This method is faster than tolerance-based prune and
     /// useful for narrow-band level set applications where inactive
-    /// values are limited to either an inside or outside value.
+    /// values are limited to either an inside or an outside value.
     void pruneLevelSet();
 
     /// @brief Add the given leaf node to this tree, creating a new branch if necessary.
@@ -430,8 +472,7 @@ public:
     /// @brief Add a tile containing voxel (x, y, z) at the specified tree level,
     /// creating a new branch if necessary.  Delete any existing lower-level nodes
     /// that contain (x, y, z).
-    /// @note @c Level must be greater than zero (i.e., the level of leaf nodes)
-    /// and less than this tree's depth.
+    /// @note @a level must be less than this tree's depth.
     void addTile(Index level, const Coord& xyz, const ValueType& value, bool active);
 
     /// @brief Return a pointer to the node of type @c NodeT that contains voxel (x, y, z)
@@ -441,22 +482,29 @@ public:
     template<typename NodeT>
     NodeT* stealNode(const Coord& xyz, const ValueType& value, bool active);
 
-    /// @brief @return the leaf node that contains voxel (x, y, z) and
-    /// if it doesn't exist, create it, but preserve the values and
+    /// @brief Return a pointer to the leaf node that contains voxel (x, y, z).
+    /// If no such node exists, create one that preserves the values and
     /// active states of all voxels.
-    ///
-    /// Use this method to preallocate a static tree topology over which to
+    /// @details Use this method to preallocate a static tree topology over which to
     /// safely perform multithreaded processing.
     LeafNodeType* touchLeaf(const Coord& xyz);
 
-    /// @brief @return a pointer to the leaf node that contains
-    /// voxel (x, y, z) and if it doesn't exist, return NULL.
-    LeafNodeType* probeLeaf(const Coord& xyz);
+    //@{
+    /// @brief Return a pointer to the node of type @c NodeType that contains
+    /// voxel (x, y, z).  If no such node exists, return NULL.
+    template<typename NodeType> NodeType* probeNode(const Coord& xyz);
+    template<typename NodeType> const NodeType* probeConstNode(const Coord& xyz) const;
+    template<typename NodeType> const NodeType* probeNode(const Coord& xyz) const;
+    //@}
 
-    /// @brief @return a const pointer to the leaf node that contains
-    /// voxel (x, y, z) and if it doesn't exist, return NULL.
+    //@{
+    /// @brief Return a pointer to the leaf node that contains voxel (x, y, z).
+    /// If no such node exists, return NULL.
+    LeafNodeType* probeLeaf(const Coord& xyz);
     const LeafNodeType* probeConstLeaf(const Coord& xyz) const;
     const LeafNodeType* probeLeaf(const Coord& xyz) const { return this->probeConstLeaf(xyz); }
+    //@}
+
 
     //
     // Aux methods
@@ -489,8 +537,6 @@ public:
 
     /// Return this tree's background value.
     const ValueType& background() const { return mRoot.background(); }
-    /// @deprecated Use background()
-    OPENVDB_DEPRECATED ValueType getBackground() const { return mRoot.background(); }
     /// Replace this tree's background value.
     void setBackground(const ValueType& background) { mRoot.setBackground(background); }
 
@@ -500,28 +546,28 @@ public:
     /// @brief Set the values of all inactive voxels and tiles of a narrow-band
     /// level set from the signs of the active voxels, setting outside values to
     /// +background and inside values to -background.
-    ///
-    /// @note This method should only be used on closed, narrow-band level sets!
+    /// @warning This method should only be used on closed, narrow-band level sets.
     void signedFloodFill() { mRoot.signedFloodFill(); }
 
     /// @brief Set the values of all inactive voxels and tiles of a narrow-band
     /// level set from the signs of the active voxels, setting exterior values to
     /// @a outside and interior values to @a inside.  Set the background value
     /// of this tree to @a outside.
-    ///
-    /// @note This method should only be used on closed, narrow-band level sets!
+    /// @warning This method should only be used on closed, narrow-band level sets.
     void signedFloodFill(const ValueType& outside, const ValueType& inside);
 
-    /// Move child nodes from the other tree into this tree wherever those nodes
-    /// correspond to constant-value tiles in this tree, and replace leaf-level
-    /// inactive voxels in this tree with corresponding voxels in the other tree
-    /// that are active.
-    /// @note This operation always empties the other tree.
-    void merge(Tree& other);
-
-    /// Turns active tiles into dense voxels, i.e. active leaf nodes
+    /// Densify active tiles, i.e., replace them with leaf-level active voxels.
     void voxelizeActiveTiles();
 
+    /// @brief Efficiently merge another tree into this tree using one of several schemes.
+    /// @details This operation is primarily intended to combine trees that are mostly
+    /// non-overlapping (for example, intermediate trees from computations that are
+    /// parallelized across disjoint regions of space).
+    /// @note This operation is not guaranteed to produce an optimally sparse tree.
+    /// Follow merge() with prune() for optimal sparseness.
+    /// @warning This operation always empties the other tree.
+    void merge(Tree& other, MergePolicy = MERGE_ACTIVE_STATES);
+
     /// @brief Union this tree's set of active values with the active values
     /// of the other tree, whose @c ValueType may be different.
     /// @details The resulting state of a value is active if the corresponding value
@@ -537,6 +583,35 @@ public:
     /// are marked as active in this tree but left with their original values.
     template<typename OtherRootNodeType>
     void topologyUnion(const Tree<OtherRootNodeType>& other);
+   
+    /// @brief Intersects this tree's set of active values with the active values
+    /// of the other tree, whose @c ValueType may be different.
+    /// @details The resulting state of a value is active only if the corresponding 
+    /// value was already active AND if it is active in the other tree. Also, a
+    /// resulting value maps to a voxel if the corresponding value
+    /// already mapped to an active voxel in either of the two grids
+    /// and it maps to an active tile or voxel in the other grid.
+    ///
+    /// @note This operation can delete branches in this grid if they
+    /// overlap with inactive tiles in the other grid. Likewise active
+    /// voxels can be turned into unactive voxels resulting in leaf
+    /// nodes with no active values. Thus, it is recommended to
+    /// subsequently call prune. 
+    template<typename OtherRootNodeType>
+    void topologyIntersection(const Tree<OtherRootNodeType>& other);
+
+    /// @brief Difference this tree's set of active values with the active values
+    /// of the other tree, whose @c ValueType may be different. So a
+    /// resulting voxel will be active only if the original voxel is
+    /// active in this tree and inactive in the other tree.
+    ///
+    /// @note This operation can delete branches in this grid if they
+    /// overlap with active tiles in the other grid. Likewise active
+    /// voxels can be turned into inactive voxels resulting in leaf
+    /// nodes with no active values. Thus, it is recommended to
+    /// subsequently call prune.
+    template<typename OtherRootNodeType>
+    void topologyDifference(const Tree<OtherRootNodeType>& other);
 
     /*! For a given function @c f, use sparse traversal to compute <tt>f(this, other)</tt>
      *  over all corresponding pairs of values (tile or voxel) of this tree and the other tree
@@ -1003,16 +1078,37 @@ protected:
 }; // end of Tree class
 
 
-/// Tree4<T, N1, N2, N3>::Type is the type of a four-level tree
+/// @brief Tree3<T, N1, N2>::Type is the type of a three-level tree
+/// (Root, Internal, Leaf) with value type T and
+/// internal and leaf node log dimensions N1 and N2, respectively.
+/// @note This is NOT the standard tree configuration (Tree4 is).
+template<typename T, Index N1, Index N2>
+struct Tree3 {
+    typedef Tree<RootNode<InternalNode<LeafNode<T, N2>, N1> > > Type;
+};
+
+
+/// @brief Tree4<T, N1, N2, N3>::Type is the type of a four-level tree
 /// (Root, Internal, Internal, Leaf) with value type T and
 /// internal and leaf node log dimensions N1, N2 and N3, respectively.
-/// This is the standard tree configuration.
+/// @note This is the standard tree configuration.
 template<typename T, Index N1, Index N2, Index N3>
 struct Tree4 {
     typedef Tree<RootNode<InternalNode<InternalNode<LeafNode<T, N3>, N2>, N1> > > Type;
 };
 
 
+/// @brief Tree5<T, N1, N2, N3, N4>::Type is the type of a five-level tree
+/// (Root, Internal, Internal, Internal, Leaf) with value type T and
+/// internal and leaf node log dimensions N1, N2, N3 and N4, respectively.
+/// @note This is NOT the standard tree configuration (Tree4 is).
+template<typename T, Index N1, Index N2, Index N3, Index N4>
+struct Tree5 {
+    typedef Tree<RootNode<InternalNode<InternalNode<InternalNode<LeafNode<T, N4>, N3>, N2>, N1> > >
+        Type;
+};
+
+
 ////////////////////////////////////////
 
 
@@ -1358,26 +1454,20 @@ Tree<RootNodeType>::setValueOn(const Coord& xyz, const ValueType& value)
 
 
 template<typename RootNodeType>
+template<typename ModifyOp>
 inline void
-Tree<RootNodeType>::setValueOnMin(const Coord& xyz, const ValueType& value)
+Tree<RootNodeType>::modifyValue(const Coord& xyz, const ModifyOp& op)
 {
-    mRoot.setValueOnMin(xyz, value);
+    mRoot.modifyValue(xyz, op);
 }
 
 
 template<typename RootNodeType>
+template<typename ModifyOp>
 inline void
-Tree<RootNodeType>::setValueOnMax(const Coord& xyz, const ValueType& value)
+Tree<RootNodeType>::modifyValueAndActiveState(const Coord& xyz, const ModifyOp& op)
 {
-    mRoot.setValueOnMax(xyz, value);
-}
-
-
-template<typename RootNodeType>
-inline void
-Tree<RootNodeType>::setValueOnSum(const Coord& xyz, const ValueType& value)
-{
-    mRoot.setValueOnSum(xyz, value);
+    mRoot.modifyValueAndActiveState(xyz, op);
 }
 
 
@@ -1445,6 +1535,7 @@ Tree<RootNodeType>::addTile(Index level, const Coord& xyz,
     mRoot.addTile(level, xyz, value, active);
 }
 
+
 template<typename RootNodeType>
 template<typename NodeT>
 inline NodeT*
@@ -1454,6 +1545,7 @@ Tree<RootNodeType>::stealNode(const Coord& xyz, const ValueType& value, bool act
     return mRoot.template stealNode<NodeT>(xyz, value, active);
 }
 
+
 template<typename RootNodeType>
 inline typename RootNodeType::LeafNodeType*
 Tree<RootNodeType>::touchLeaf(const Coord& xyz)
@@ -1478,6 +1570,33 @@ Tree<RootNodeType>::probeConstLeaf(const Coord& xyz) const
 }
 
 
+template<typename RootNodeType>
+template<typename NodeType>
+inline NodeType*
+Tree<RootNodeType>::probeNode(const Coord& xyz)
+{
+    return mRoot.template probeNode<NodeType>(xyz);
+}
+
+
+template<typename RootNodeType>
+template<typename NodeType>
+inline const NodeType*
+Tree<RootNodeType>::probeNode(const Coord& xyz) const
+{
+    return this->template probeConstNode<NodeType>(xyz);
+}
+
+
+template<typename RootNodeType>
+template<typename NodeType>
+inline const NodeType*
+Tree<RootNodeType>::probeConstNode(const Coord& xyz) const
+{
+    return mRoot.template probeConstNode<NodeType>(xyz);
+}
+
+
 ////////////////////////////////////////
 
 
@@ -1519,20 +1638,27 @@ Tree<RootNodeType>::getBackgroundValue() const
 
 template<typename RootNodeType>
 inline void
-Tree<RootNodeType>::merge(Tree& other)
+Tree<RootNodeType>::voxelizeActiveTiles()
 {
     this->clearAllAccessors();
-    other.clearAllAccessors();
-    mRoot.merge(other.mRoot);
+    mRoot.voxelizeActiveTiles();
 }
 
 
 template<typename RootNodeType>
 inline void
-Tree<RootNodeType>::voxelizeActiveTiles()
+Tree<RootNodeType>::merge(Tree& other, MergePolicy policy)
 {
     this->clearAllAccessors();
-    mRoot.voxelizeActiveTiles();
+    other.clearAllAccessors();
+    switch (policy) {
+        case MERGE_ACTIVE_STATES:
+            mRoot.template merge<MERGE_ACTIVE_STATES>(other.mRoot); break;
+        case MERGE_NODES:
+            mRoot.template merge<MERGE_NODES>(other.mRoot); break;
+        case MERGE_ACTIVE_STATES_AND_NODES:
+            mRoot.template merge<MERGE_ACTIVE_STATES_AND_NODES>(other.mRoot); break;
+    }
 }
 
 
@@ -1545,6 +1671,23 @@ Tree<RootNodeType>::topologyUnion(const Tree<OtherRootNodeType>& other)
     mRoot.topologyUnion(other.getRootNode());
 }
 
+template<typename RootNodeType>
+template<typename OtherRootNodeType>
+inline void
+Tree<RootNodeType>::topologyIntersection(const Tree<OtherRootNodeType>& other)
+{
+    this->clearAllAccessors();
+    mRoot.topologyIntersection(other.getRootNode());
+}
+
+template<typename RootNodeType>
+template<typename OtherRootNodeType>
+inline void
+Tree<RootNodeType>::topologyDifference(const Tree<OtherRootNodeType>& other)
+{
+    this->clearAllAccessors();
+    mRoot.topologyDifference(other.getRootNode());
+}
 
 ////////////////////////////////////////
 
@@ -1773,8 +1916,7 @@ Tree<RootNodeType>::treeType()
             ostr << "_" << dims[i];
         }
         Name* s = new Name(ostr.str());
-        sTypeName.compare_and_swap(s, NULL);
-        if (sTypeName != s) delete s;
+        if (sTypeName.compare_and_swap(s, NULL) != NULL) delete s;
     }
     return *sTypeName;
 }
@@ -1807,31 +1949,25 @@ template<typename RootNodeType>
 inline bool
 Tree<RootNodeType>::evalLeafBoundingBox(CoordBBox& bbox) const
 {
-    if (this->empty()) {
-        bbox = CoordBBox(); // return default bbox
-        return false;// empty
-    }
+    bbox.reset(); // default invalid bbox
+    
+    if (this->empty()) return false;  // empty
 
-    bbox.min() =  Coord::max();
-    bbox.max() = -Coord::max();
+    mRoot.evalActiveBoundingBox(bbox, false);
 
-    Coord ijk;
-    for (LeafCIter bIter(*this); bIter; ++bIter) {
-        bIter->getOrigin(ijk);
-        bbox.expand(ijk);
-    }
-    bbox.max() += Coord(LeafNodeType::dim()-1);
-    return true; // not empty
+    return true;// not empty
 }
 
 template<typename RootNodeType>
 inline bool
 Tree<RootNodeType>::evalActiveVoxelBoundingBox(CoordBBox& bbox) const
 {
-    bbox = CoordBBox(); // default invalid bbox
+    
+    bbox.reset(); // default invalid bbox
+    
     if (this->empty()) return false;  // empty
 
-    mRoot.evalActiveVoxelBoundingBox(bbox);
+    mRoot.evalActiveBoundingBox(bbox, true);
 
     return true;// not empty
 }
