@@ -40,26 +40,33 @@ public:
     CPPUNIT_TEST_SUITE(TestMaps);
     CPPUNIT_TEST(testTranslation);
     CPPUNIT_TEST(testRotation);
+    CPPUNIT_TEST(testScaleDefault);
     CPPUNIT_TEST(testScaleTranslate);
     CPPUNIT_TEST(testUniformScaleTranslate);
     CPPUNIT_TEST(testDecomposition);
     CPPUNIT_TEST(testFrustum);
     CPPUNIT_TEST(testCalcBoundingBox);
     CPPUNIT_TEST(testApproxInverse);
+    CPPUNIT_TEST(testUniformScale);
+    CPPUNIT_TEST(testJacobians);
     CPPUNIT_TEST_SUITE_END();
 
     void testTranslation();
     void testRotation();
+    void testScaleDefault();
     void testScaleTranslate();
     void testUniformScaleTranslate();
     void testDecomposition();
     void testFrustum();
     void testCalcBoundingBox();
     void testApproxInverse();
+    void testUniformScale();
+    void testJacobians();
     //void testIsType();
 };
 
 CPPUNIT_TEST_SUITE_REGISTRATION(TestMaps);
+
 void
 TestMaps::testApproxInverse()
 {
@@ -109,8 +116,32 @@ TestMaps::testApproxInverse()
     
   
 }
+
+
+void
+TestMaps::testUniformScale()
+{
+    using namespace openvdb::math;
+
+    AffineMap map;
     
-    
+    CPPUNIT_ASSERT(map.hasUniformScale());
+
+    // Apply uniform scale: should still have square voxels
+    map.accumPreScale(Vec3d(2, 2, 2));
+
+    CPPUNIT_ASSERT(map.hasUniformScale());
+
+    // Apply a rotation, should still have squaure voxels.
+    map.accumPostRotation(X_AXIS, 2.5);
+
+    CPPUNIT_ASSERT(map.hasUniformScale());
+
+    // non uniform scaling will stretch the voxels
+    map.accumPostScale(Vec3d(1, 3, 1) );
+
+    CPPUNIT_ASSERT(!map.hasUniformScale());
+}
 
 void
 TestMaps::testTranslation()
@@ -151,8 +182,60 @@ TestMaps::testTranslation()
     CPPUNIT_ASSERT_DOUBLES_EQUAL(result(0), 1, TOL);
     CPPUNIT_ASSERT_DOUBLES_EQUAL(result(1), 0, TOL);
     CPPUNIT_ASSERT_DOUBLES_EQUAL(result(2), 0, TOL);
+
+    /// Jacobian Transpose
+    result = translate_by_two->applyJT(translate_by_two->applyIJT(unit));
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(result(0), unit(0), TOL);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(result(1), unit(1), TOL);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(result(2), unit(2), TOL);
+
+
+    MapBase::Ptr inverse = translation->inverseMap();
+    CPPUNIT_ASSERT(inverse->type() == TranslationMap::mapType());
+    // apply the map forward and the inverse map back
+    result = inverse->applyMap(translation->applyMap(unit));
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(result(0), 1, TOL);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(result(1), 0, TOL);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(result(2), 0, TOL);
+
+  
 }
 
+void
+TestMaps::testScaleDefault()
+{
+    using namespace openvdb::math;
+
+    double TOL = 1e-7;
+
+    // testing default constructor
+    // should be the identity
+    ScaleMap::Ptr  scale(new ScaleMap());
+    Vec3d unit(1, 1, 1);
+    
+    Vec3d result = scale->applyMap(unit);
+    
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(unit(0), result(0), TOL);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(unit(1), result(1), TOL);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(unit(2), result(2), TOL);
+    
+    result = scale->applyInverseMap(unit);
+
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(unit(0), result(0), TOL);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(unit(1), result(1), TOL);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(unit(2), result(2), TOL);
+
+    
+    MapBase::Ptr inverse = scale->inverseMap();
+    CPPUNIT_ASSERT(inverse->type() == ScaleMap::mapType());
+    // apply the map forward and the inverse map back
+    result = inverse->applyMap(scale->applyMap(unit));
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(result(0), unit(0), TOL);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(result(1), unit(1), TOL);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(result(2), unit(2), TOL);
+
+    
+}
 
 void
 TestMaps::testRotation()
@@ -195,6 +278,22 @@ TestMaps::testRotation()
     CPPUNIT_ASSERT_DOUBLES_EQUAL(0, result(0), TOL);
     CPPUNIT_ASSERT_DOUBLES_EQUAL(sqrt(2.)/2, result(1), TOL);
     CPPUNIT_ASSERT_DOUBLES_EQUAL(sqrt(2.)/2, result(2), TOL);
+
+    /// Jacobian Transpose
+    result = rotation_two->applyJT(rotation_two->applyIJT(unit));
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(result(0), unit(0), TOL);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(result(1), unit(1), TOL);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(result(2), unit(2), TOL);
+
+
+    // Test inverse map
+    MapBase::Ptr inverse = rotation->inverseMap();
+    CPPUNIT_ASSERT(inverse->type() == UnitaryMap::mapType());
+    // apply the map forward and the inverse map back
+    result = inverse->applyMap(rotation->applyMap(unit));
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(result(0), unit(0), TOL);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(result(1), unit(1), TOL);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(result(2), unit(2), TOL);
 }
 
 
@@ -241,6 +340,23 @@ TestMaps::testScaleTranslate()
     CPPUNIT_ASSERT_DOUBLES_EQUAL(1, result(1), TOL);
     CPPUNIT_ASSERT_DOUBLES_EQUAL(0, result(2), TOL);
 
+
+    /// Jacobian Transpose
+    result = scaleAndTranslate->applyJT(scaleAndTranslate->applyIJT(unit));
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(result(0), unit(0), TOL);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(result(1), unit(1), TOL);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(result(2), unit(2), TOL);
+
+
+    // Test inverse map
+    MapBase::Ptr inverse = scaleAndTranslate->inverseMap();
+    CPPUNIT_ASSERT(inverse->type() == ScaleTranslateMap::mapType());
+    // apply the map forward and the inverse map back
+    result = inverse->applyMap(scaleAndTranslate->applyMap(unit));
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(result(0), unit(0), TOL);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(result(1), unit(1), TOL);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(result(2), unit(2), TOL);
+
 }
 
 
@@ -286,6 +402,25 @@ TestMaps::testUniformScaleTranslate()
     CPPUNIT_ASSERT_DOUBLES_EQUAL(0, result(0), TOL);
     CPPUNIT_ASSERT_DOUBLES_EQUAL(1, result(1), TOL);
     CPPUNIT_ASSERT_DOUBLES_EQUAL(0, result(2), TOL);
+
+
+    /// Jacobian Transpose
+    result = scaleAndTranslate->applyJT(scaleAndTranslate->applyIJT(unit));
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(result(0), unit(0), TOL);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(result(1), unit(1), TOL);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(result(2), unit(2), TOL);
+
+
+
+    // Test inverse map
+    MapBase::Ptr inverse = scaleAndTranslate->inverseMap();
+    CPPUNIT_ASSERT(inverse->type() == UniformScaleTranslateMap::mapType());
+    // apply the map forward and the inverse map back
+    result = inverse->applyMap(scaleAndTranslate->applyMap(unit));
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(result(0), unit(0), TOL);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(result(1), unit(1), TOL);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(result(2), unit(2), TOL);
+
 }
 
 
@@ -541,6 +676,158 @@ TestMaps::testCalcBoundingBox()
          openvdb::util::calculateBounds<NonlinearFrustumMap>(*map, center, radius, voxel_bbox);
      }
 }
+void
+TestMaps::testJacobians()
+{
+    using namespace openvdb::math;
+    const double TOL = 1e-7;
+    {
+        AffineMap affine;
+        
+        const int n = 10;
+        const double dtheta = M_PI / n;
+        
+        const Vec3d test(1,2,3);
+        const Vec3d origin(0,0,0);
+        
+        for (int i = 0; i < n; ++i) {
+            double theta = i * dtheta;
+            
+            affine.accumPostRotation(X_AXIS, theta);
+            
+            Vec3d result = affine.applyJacobian(test);
+            Vec3d expected = affine.applyMap(test) - affine.applyMap(origin);
+            
+            CPPUNIT_ASSERT_DOUBLES_EQUAL(result(0), expected(0), TOL);
+            CPPUNIT_ASSERT_DOUBLES_EQUAL(result(1), expected(1), TOL);
+            CPPUNIT_ASSERT_DOUBLES_EQUAL(result(2), expected(2), TOL);
+            
+            Vec3d tmp = affine.applyInverseJacobian(result);
+            
+            CPPUNIT_ASSERT_DOUBLES_EQUAL(tmp(0), test(0), TOL);
+            CPPUNIT_ASSERT_DOUBLES_EQUAL(tmp(1), test(1), TOL);
+            CPPUNIT_ASSERT_DOUBLES_EQUAL(tmp(2), test(2), TOL);
+        }
+    }
+
+    {
+        UniformScaleMap scale(3);
+        const Vec3d test(1,2,3);
+        const Vec3d origin(0,0,0);
+        
+        
+        Vec3d result = scale.applyJacobian(test);
+        Vec3d expected = scale.applyMap(test) - scale.applyMap(origin);
+        
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(result(0), expected(0), TOL);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(result(1), expected(1), TOL);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(result(2), expected(2), TOL);
+        
+        Vec3d tmp = scale.applyInverseJacobian(result);
+        
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(tmp(0), test(0), TOL);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(tmp(1), test(1), TOL);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(tmp(2), test(2), TOL);
+    }
+        
+    {
+        ScaleMap scale(Vec3d(1,2,3));
+        const Vec3d test(1,2,3);
+        const Vec3d origin(0,0,0);
+        
+        
+        Vec3d result = scale.applyJacobian(test);
+        Vec3d expected = scale.applyMap(test) - scale.applyMap(origin);
+        
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(result(0), expected(0), TOL);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(result(1), expected(1), TOL);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(result(2), expected(2), TOL);
+        
+        Vec3d tmp = scale.applyInverseJacobian(result);
+        
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(tmp(0), test(0), TOL);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(tmp(1), test(1), TOL);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(tmp(2), test(2), TOL);
+    }
+    {
+        TranslationMap map(Vec3d(1,2,3));
+        const Vec3d test(1,2,3);
+        const Vec3d origin(0,0,0);
+        
+        
+        Vec3d result = map.applyJacobian(test);
+        Vec3d expected = map.applyMap(test) - map.applyMap(origin);
+        
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(result(0), expected(0), TOL);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(result(1), expected(1), TOL);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(result(2), expected(2), TOL);
+        
+        Vec3d tmp = map.applyInverseJacobian(result);
+        
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(tmp(0), test(0), TOL);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(tmp(1), test(1), TOL);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(tmp(2), test(2), TOL);
+    }
+    {
+        ScaleTranslateMap map(Vec3d(1,2,3), Vec3d(3,5,4));
+        const Vec3d test(1,2,3);
+        const Vec3d origin(0,0,0);
+        
+        
+        Vec3d result = map.applyJacobian(test);
+        Vec3d expected = map.applyMap(test) - map.applyMap(origin);
+        
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(result(0), expected(0), TOL);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(result(1), expected(1), TOL);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(result(2), expected(2), TOL);
+        
+        Vec3d tmp = map.applyInverseJacobian(result);
+        
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(tmp(0), test(0), TOL);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(tmp(1), test(1), TOL);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(tmp(2), test(2), TOL);
+    }
+    {
+        
+        openvdb::BBoxd bbox(Vec3d(0), Vec3d(100));
+        NonlinearFrustumMap frustum(bbox, 1./6., 5);
+        /// frustum will have depth, far plane - near plane = 5
+        /// the frustum has width 1 in the front and 6 in the back
+        
+        Vec3d trans(2,2,2);
+        NonlinearFrustumMap::Ptr map =
+            boost::static_pointer_cast<NonlinearFrustumMap, MapBase>(
+            frustum.preScale(Vec3d(10,10,10))->postTranslate(trans));
+
+        
+        const Vec3d test(1,2,3);
+        const Vec3d origin(0, 0, 0);
+        
+        // these two drop down to just the linear part
+        Vec3d lresult = map->applyJacobian(test);
+        Vec3d ltmp = map->applyInverseJacobian(lresult);
+        
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(ltmp(0), test(0), TOL);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(ltmp(1), test(1), TOL);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(ltmp(2), test(2), TOL);
+
+        Vec3d isloc(4,5,6);
+        // these two drop down to just the linear part
+        Vec3d result = map->applyJacobian(test, isloc);
+        Vec3d tmp = map->applyInverseJacobian(result, isloc);
+        
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(tmp(0), test(0), TOL);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(tmp(1), test(1), TOL);
+        CPPUNIT_ASSERT_DOUBLES_EQUAL(tmp(2), test(2), TOL);
+
+
+        
+    }
+    
+     
+}
+
+
 
 // Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
