@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -44,11 +44,13 @@ public:
     CPPUNIT_TEST(testLinearTransform);
     CPPUNIT_TEST(testTransformEquality);
     CPPUNIT_TEST(testBackwardCompatibility);
+    CPPUNIT_TEST(testIsIdentity);
     CPPUNIT_TEST_SUITE_END();
 
     void testLinearTransform();
     void testTransformEquality();
     void testBackwardCompatibility();
+    void testIsIdentity();
 };
 
 CPPUNIT_TEST_SUITE_REGISTRATION(TestTransform);
@@ -455,7 +457,7 @@ TestTransform::testBackwardCompatibility()
     t->read(ss);
 
     CPPUNIT_ASSERT_EQUAL(math::AffineMap::mapType(), t->baseMap()->type());
-
+    
     voxelSize = t->voxelSize();
 
     CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, voxelSize[0], TOL);
@@ -470,6 +472,38 @@ TestTransform::testBackwardCompatibility()
 }
 
 
+void
+TestTransform::testIsIdentity()
+{
+    using namespace openvdb;
+    math::Transform::Ptr t = math::Transform::createLinearTransform(1.0);
+    
+    CPPUNIT_ASSERT(t->isIdentity());
+    
+    t->preScale(Vec3d(2,2,2));
+    
+    CPPUNIT_ASSERT(!t->isIdentity());
+    
+    t->preScale(Vec3d(0.5,0.5,0.5));
+    CPPUNIT_ASSERT(t->isIdentity());
+    
+    BBoxd bbox(math::Vec3d(-5,-5,0), Vec3d(5,5,10));
+    math::Transform::Ptr f = math::Transform::createFrustumTransform(bbox,
+                                                                     /*taper*/ 1,
+                                                                     /*depth*/ 1,
+                                                                     /*voxel size*/ 1);
+    f->preScale(Vec3d(10,10,10));
+
+    CPPUNIT_ASSERT(f->isIdentity());
+
+    // rotate by PI/2
+    f->postRotate(std::atan(1.0)*2, math::Y_AXIS);
+    CPPUNIT_ASSERT(!f->isIdentity());
+
+    f->postRotate(std::atan(1.0)*6, math::Y_AXIS);
+    CPPUNIT_ASSERT(f->isIdentity());
+}
+
 ////////////////////////////////////////
 
 
@@ -483,6 +517,6 @@ TestTransform::testNonlinearTransform()
 }
 */
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
