@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -237,8 +237,8 @@ TestTreeCombine::testComp(const TreeComp& comp, const ValueComp& op)
         TreeT aTree(/*background=*/one);
         aTree.setValueOn(openvdb::Coord(0, 0, 0), three);
         aTree.setValueOn(openvdb::Coord(0, 0, 1), three);
-        aTree.setValueOn(openvdb::Coord(0, 0, 2), aTree.getBackground());
-        aTree.setValueOn(openvdb::Coord(0, 1, 2), aTree.getBackground());
+        aTree.setValueOn(openvdb::Coord(0, 0, 2), aTree.background());
+        aTree.setValueOn(openvdb::Coord(0, 1, 2), aTree.background());
         aTree.setValueOff(openvdb::Coord(1, 0, 0), three);
         aTree.setValueOff(openvdb::Coord(1, 0, 1), three);
 
@@ -293,8 +293,8 @@ TestTreeCombine::testComp(const TreeComp& comp, const ValueComp& op)
         TreeT aTree(/*bg=*/one);
         aTree.setValueOn(openvdb::Coord(0, 0, 0), three);
         aTree.setValueOn(openvdb::Coord(0, 0, 1), three);
-        aTree.setValueOn(openvdb::Coord(0, 0, 2), aTree.getBackground());
-        aTree.setValueOn(openvdb::Coord(0, 1, 2), aTree.getBackground());
+        aTree.setValueOn(openvdb::Coord(0, 0, 2), aTree.background());
+        aTree.setValueOn(openvdb::Coord(0, 1, 2), aTree.background());
         aTree.setValueOff(openvdb::Coord(1, 0, 0), three);
         aTree.setValueOff(openvdb::Coord(1, 0, 1), three);
 
@@ -464,8 +464,8 @@ TestTreeCombine::testCompRepl()
         TreeT aTree(/*bg=*/one);
         aTree.setValueOn(openvdb::Coord(0, 0, 0), three);
         aTree.setValueOn(openvdb::Coord(0, 0, 1), three);
-        aTree.setValueOn(openvdb::Coord(0, 0, 2), aTree.getBackground());
-        aTree.setValueOn(openvdb::Coord(0, 1, 2), aTree.getBackground());
+        aTree.setValueOn(openvdb::Coord(0, 0, 2), aTree.background());
+        aTree.setValueOn(openvdb::Coord(0, 1, 2), aTree.background());
         aTree.setValueOff(openvdb::Coord(1, 0, 0), three);
         aTree.setValueOff(openvdb::Coord(1, 0, 1), three);
 
@@ -520,8 +520,8 @@ TestTreeCombine::testCompRepl()
         TreeT aTree(/*background=*/one);
         aTree.setValueOn(openvdb::Coord(0, 0, 0), three);
         aTree.setValueOn(openvdb::Coord(0, 0, 1), three);
-        aTree.setValueOn(openvdb::Coord(0, 0, 2), aTree.getBackground());
-        aTree.setValueOn(openvdb::Coord(0, 1, 2), aTree.getBackground());
+        aTree.setValueOn(openvdb::Coord(0, 0, 2), aTree.background());
+        aTree.setValueOn(openvdb::Coord(0, 1, 2), aTree.background());
         aTree.setValueOff(openvdb::Coord(1, 0, 0), three);
         aTree.setValueOff(openvdb::Coord(1, 0, 1), three);
 
@@ -723,6 +723,6 @@ TestTreeCombine::visitCsg(const TreeT& aInputTree, const TreeT& bInputTree,
     return aTree;
 }
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
