@@ -53,7 +53,7 @@ TestVec2Metadata::testVec2i()
 {
     using namespace openvdb;
 
-    Metadata::Ptr m(new Vec2IMetadata(openvdb::math::Vec2i(1, 1)));
+    Metadata::Ptr m(new Vec2IMetadata(openvdb::Vec2i(1, 1)));
     Metadata::Ptr m2 = m->copy();
 
     CPPUNIT_ASSERT(dynamic_cast<Vec2IMetadata*>(m.get()) != 0);
@@ -63,14 +63,14 @@ TestVec2Metadata::testVec2i()
     CPPUNIT_ASSERT(m2->typeName().compare("vec2i") == 0);
 
     Vec2IMetadata *s = dynamic_cast<Vec2IMetadata*>(m.get());
-    CPPUNIT_ASSERT(s->value() == openvdb::math::Vec2i(1, 1));
-    s->value() = openvdb::math::Vec2i(2, 2);
-    CPPUNIT_ASSERT(s->value() == openvdb::math::Vec2i(2, 2));
+    CPPUNIT_ASSERT(s->value() == openvdb::Vec2i(1, 1));
+    s->value() = openvdb::Vec2i(2, 2);
+    CPPUNIT_ASSERT(s->value() == openvdb::Vec2i(2, 2));
 
     m2->copy(*s);
 
     s = dynamic_cast<Vec2IMetadata*>(m2.get());
-    CPPUNIT_ASSERT(s->value() == openvdb::math::Vec2i(2, 2));
+    CPPUNIT_ASSERT(s->value() == openvdb::Vec2i(2, 2));
 }
 
 void
@@ -78,7 +78,7 @@ TestVec2Metadata::testVec2s()
 {
     using namespace openvdb;
 
-    Metadata::Ptr m(new Vec2SMetadata(openvdb::math::Vec2s(1, 1)));
+    Metadata::Ptr m(new Vec2SMetadata(openvdb::Vec2s(1, 1)));
     Metadata::Ptr m2 = m->copy();
 
     CPPUNIT_ASSERT(dynamic_cast<Vec2SMetadata*>(m.get()) != 0);
@@ -88,14 +88,14 @@ TestVec2Metadata::testVec2s()
     CPPUNIT_ASSERT(m2->typeName().compare("vec2s") == 0);
 
     Vec2SMetadata *s = dynamic_cast<Vec2SMetadata*>(m.get());
-    CPPUNIT_ASSERT(s->value() == openvdb::math::Vec2s(1, 1));
-    s->value() = openvdb::math::Vec2s(2, 2);
-    CPPUNIT_ASSERT(s->value() == openvdb::math::Vec2s(2, 2));
+    CPPUNIT_ASSERT(s->value() == openvdb::Vec2s(1, 1));
+    s->value() = openvdb::Vec2s(2, 2);
+    CPPUNIT_ASSERT(s->value() == openvdb::Vec2s(2, 2));
 
     m2->copy(*s);
 
     s = dynamic_cast<Vec2SMetadata*>(m2.get());
-    CPPUNIT_ASSERT(s->value() == openvdb::math::Vec2s(2, 2));
+    CPPUNIT_ASSERT(s->value() == openvdb::Vec2s(2, 2));
 }
 
 void
@@ -103,7 +103,7 @@ TestVec2Metadata::testVec2d()
 {
     using namespace openvdb;
 
-    Metadata::Ptr m(new Vec2DMetadata(openvdb::math::Vec2d(1, 1)));
+    Metadata::Ptr m(new Vec2DMetadata(openvdb::Vec2d(1, 1)));
     Metadata::Ptr m2 = m->copy();
 
     CPPUNIT_ASSERT(dynamic_cast<Vec2DMetadata*>(m.get()) != 0);
@@ -113,14 +113,14 @@ TestVec2Metadata::testVec2d()
     CPPUNIT_ASSERT(m2->typeName().compare("vec2d") == 0);
 
     Vec2DMetadata *s = dynamic_cast<Vec2DMetadata*>(m.get());
-    CPPUNIT_ASSERT(s->value() == openvdb::math::Vec2d(1, 1));
-    s->value() = openvdb::math::Vec2d(2, 2);
-    CPPUNIT_ASSERT(s->value() == openvdb::math::Vec2d(2, 2));
+    CPPUNIT_ASSERT(s->value() == openvdb::Vec2d(1, 1));
+    s->value() = openvdb::Vec2d(2, 2);
+    CPPUNIT_ASSERT(s->value() == openvdb::Vec2d(2, 2));
 
     m2->copy(*s);
 
     s = dynamic_cast<Vec2DMetadata*>(m2.get());
-    CPPUNIT_ASSERT(s->value() == openvdb::math::Vec2d(2, 2));
+    CPPUNIT_ASSERT(s->value() == openvdb::Vec2d(2, 2));
 }
 
 // Copyright (c) 2012-2013 DreamWorks Animation LLC
