@@ -28,15 +28,13 @@
 //
 ///////////////////////////////////////////////////////////////////////////
 
-#include <vector>
 #include <cppunit/extensions/HelperMacros.h>
 
-#include <openvdb/openvdb.h>
-#include <openvdb/Exceptions.h>
-
 #include <openvdb/tools/VolumeToMesh.h>
-#include <openvdb/util/Util.h>
+#include <openvdb/Exceptions.h>
+#include <openvdb/tree/LeafManager.h>
 
+#include <vector>
 
 class TestVolumeToMesh: public CppUnit::TestCase
 {
@@ -59,7 +57,8 @@ CPPUNIT_TEST_SUITE_REGISTRATION(TestVolumeToMesh);
 void
 TestVolumeToMesh::testAuxData()
 {
-    openvdb::FloatTree::Ptr tree(new openvdb::FloatTree(0));
+    typedef openvdb::tree::Tree4<float, 5, 4, 3>::Type Tree543f;
+    Tree543f::Ptr tree(new Tree543f(0));
 
     // create one voxel with 3 upwind edges (that have a sign change)
     tree->setValue(openvdb::Coord(0,0,0), -1);
@@ -67,34 +66,38 @@ TestVolumeToMesh::testAuxData()
     tree->setValue(openvdb::Coord(0,1,0),  1);
     tree->setValue(openvdb::Coord(0,0,1),  1);
 
+    typedef openvdb::tree::LeafManager<const Tree543f> LeafManager;
+    
+    LeafManager leafs(*tree);
+    
+    CPPUNIT_ASSERT(openvdb::tools::internal::needsActiveVoxePadding(leafs, 0.0, 1.0));
 
-    openvdb::tools::internal::LeafCPtrList<openvdb::FloatTree> leafs(*tree);
-
-    openvdb::tools::internal::AuxiliaryData<openvdb::FloatTree> init(*tree, leafs, 0.0);
-
-    init.runParallel();
-
-    CPPUNIT_ASSERT(init.auxTree()->activeVoxelCount() == 7);
-    CPPUNIT_ASSERT(init.edgeTree()->activeVoxelCount() == 1);
-
-    int flags = int(init.edgeTree()->getValue(openvdb::Coord(0,0,0)));
+    openvdb::tools::internal::SignData<Tree543f, LeafManager> op(*tree, leafs, 0.0);
+    op.run();
+    
+    CPPUNIT_ASSERT(op.signTree()->activeVoxelCount() == 1);
+    CPPUNIT_ASSERT(op.signTree()->activeVoxelCount() == op.idxTree()->activeVoxelCount());
+      
+        
+    int flags = int(op.signTree()->getValue(openvdb::Coord(0,0,0)));
 
     CPPUNIT_ASSERT(bool(flags & openvdb::tools::internal::INSIDE));
+    CPPUNIT_ASSERT(bool(flags & openvdb::tools::internal::EDGES));
     CPPUNIT_ASSERT(bool(flags & openvdb::tools::internal::XEDGE));
     CPPUNIT_ASSERT(bool(flags & openvdb::tools::internal::YEDGE));
     CPPUNIT_ASSERT(bool(flags & openvdb::tools::internal::ZEDGE));
 
 
     tree->setValueOff(openvdb::Coord(0,0,1), -1);
+    op.run();
 
-    init.runParallel();
+    CPPUNIT_ASSERT(op.signTree()->activeVoxelCount() == 1);
+    CPPUNIT_ASSERT(op.signTree()->activeVoxelCount() == op.idxTree()->activeVoxelCount());
 
-    CPPUNIT_ASSERT(init.auxTree()->activeVoxelCount() == 7);
-    CPPUNIT_ASSERT(init.edgeTree()->activeVoxelCount() == 1);
-
-    flags = int(init.edgeTree()->getValue(openvdb::Coord(0,0,0)));
+    flags = int(op.signTree()->getValue(openvdb::Coord(0,0,0)));
 
     CPPUNIT_ASSERT(bool(flags & openvdb::tools::internal::INSIDE));
+    CPPUNIT_ASSERT(bool(flags & openvdb::tools::internal::EDGES));
     CPPUNIT_ASSERT(bool(flags & openvdb::tools::internal::XEDGE));
     CPPUNIT_ASSERT(bool(flags & openvdb::tools::internal::YEDGE));
     CPPUNIT_ASSERT(!bool(flags & openvdb::tools::internal::ZEDGE));
@@ -106,7 +109,11 @@ TestVolumeToMesh::testConversion()
 {
     using namespace openvdb;
 
-    FloatGrid::Ptr grid = createGrid<FloatGrid>(/*background=*/1);
+    typedef tree::Tree4<float, 5, 4, 3>::Type Tree543f;
+    typedef Grid<Tree543f> GridType;
+
+    GridType::Ptr grid = createGrid<GridType>(/*background=*/1);
+    
     grid->fill(CoordBBox(Coord(0), Coord(7)), 0.0);
     grid->fill(CoordBBox(Coord(1), Coord(6)), -1.0);
 
