@@ -32,6 +32,8 @@
 #define OPENVDB_UNITTEST_UTIL_HAS_BEEN_INCLUDED
 
 #include <openvdb/openvdb.h>
+#include <tbb/tick_count.h>
+#include <sstream>
 
 namespace unittest_util {
 
@@ -118,6 +120,33 @@ makeSphere(const openvdb::Coord& dim, const openvdb::Vec3f& center, float radius
 
 // @todo makePlane
 
+
+////////////////////////////////////////
+
+
+class CpuTimer
+{
+public:
+    CpuTimer() {}
+
+    void start(const std::string& msg = std::string("Task"))
+    {
+        std::cerr << msg << " ... ";
+        mT0 = tbb::tick_count::now();
+    }
+
+    void stop() const
+    {
+        tbb::tick_count t1 = tbb::tick_count::now();
+        std::ostringstream ostr;
+        ostr << "completed in " << std::setprecision(3) << (t1 - mT0).seconds() << " sec\n";
+        std::cerr << ostr.str();
+    }
+
+private:
+    tbb::tick_count mT0;
+};// CpuTimer
+
 } // namespace unittest_util
 
 #endif // OPENVDB_UNITTEST_UTIL_HAS_BEEN_INCLUDED
