@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -182,7 +182,7 @@ findTangentPoints(const double g, const double xo, const double zo,
 /// @note This specialization is optimized for a frustum map
 template<>
 inline void
-calculateBounds<math::NonlinearFrustumMap>(const math::NonlinearFrustumMap& map,
+calculateBounds<math::NonlinearFrustumMap>(const math::NonlinearFrustumMap& frustum,
     const Vec3d& center, const Real radius, BBoxd& out)
 {
     // The frustum is a nonlinear map followed by a uniform scale, rotation, translation.
@@ -191,27 +191,29 @@ calculateBounds<math::NonlinearFrustumMap>(const math::NonlinearFrustumMap& map,
     // on the z=0 plane and the "camera" is located at (x=0, y=0, z=-1/g).
 
     // check that the internal map has no shear.
-    const math::AffineMap& secondMap = map.secondMap();
+    const math::AffineMap& secondMap = frustum.secondMap();
     // test if the linear part has shear or non-uniform scaling
-    if (!map.hasSimpleAffine()) {
+    if (!frustum.hasSimpleAffine()) {
+
         // In this case, we form an axis-aligned bounding box for sphere in world space
         // and find the pre_images of the corners in voxel space.  From these corners we
         // compute an axis-algined bounding box in voxel-spae
         BBoxd bounding_box(center - radius*Vec3d(1,1,1), center + radius*Vec3d(1,1,1));
-        calculateBounds<math::NonlinearFrustumMap>(map, bounding_box, out);
+        calculateBounds<math::NonlinearFrustumMap>(frustum, bounding_box, out);
         return;
     }
-
+    
     // for convenience
     Vec3d& out_min = out.min();
     Vec3d& out_max = out.max();
 
     Vec3d centerLS = secondMap.applyInverseMap(center);
     Vec3d voxelSize = secondMap.voxelSize();
-    // all the voxels have the same size since we know this is a simple affine map
-    double radiusLS = radius/ voxelSize(0);
 
-    double gamma = map.getGamma();
+    // all the voxels have the same size since we know this is a simple affine map
+    double radiusLS = radius / voxelSize(0);
+
+    double gamma = frustum.getGamma();
     double xp;
     double zp;
     double xm;
@@ -219,7 +221,7 @@ calculateBounds<math::NonlinearFrustumMap>(const math::NonlinearFrustumMap& map,
     int soln_number;
 
     // the bounding box in index space for the points in the frustum
-    const BBoxd&  bbox = map.getBBox();
+    const BBoxd&  bbox = frustum.getBBox();
     // initialize min and max
     const double x_min = bbox.min().x();
     const double y_min = bbox.min().y();
@@ -244,17 +246,26 @@ calculateBounds<math::NonlinearFrustumMap>(const math::NonlinearFrustumMap& map,
         extreme.y() = centerLS.y();
         extreme.z() = zp;
 
+        // location in world space of the tangent point
         extreme2 = secondMap.applyMap(extreme);
-        // back into voxel space
-        pre_image = map.applyInverseMap(extreme2);
+        // convert back to voxel space
+        pre_image = frustum.applyInverseMap(extreme2);
         out_max.x() = std::max(x_min, std::min(x_max, pre_image.x()));
+        
+        const Vec3d tmpPlus = extreme2;
 
         extreme.x() = xm;
         extreme.y() = centerLS.y();
         extreme.z() = zm;
+        // location in world space of the tangent point
         extreme2 = secondMap.applyMap(extreme);
-        pre_image = map.applyInverseMap(extreme2);
+
+        const Vec3d tmpMinus = extreme2;
+
+        // convert back to voxel space
+        pre_image = frustum.applyInverseMap(extreme2);
         out_min.x() = std::max(x_min, std::min(x_max, pre_image.x()));
+
     } else if (soln_number == 1) {
         // the circle was tangent at the focal point
     } else if (soln_number == 0) {
@@ -268,16 +279,23 @@ calculateBounds<math::NonlinearFrustumMap>(const math::NonlinearFrustumMap& map,
         extreme.y() = xp;
         extreme.z() = zp;
 
+        // location in world space of the tangent point
         extreme2 = secondMap.applyMap(extreme);
-
-        pre_image = map.applyInverseMap(extreme2);
+        // convert back to voxel space
+        pre_image = frustum.applyInverseMap(extreme2);
         out_max.y() = std::max(y_min, std::min(y_max, pre_image.y()));
 
+        const Vec3d tmpPlus = extreme2;
+
         extreme.x() = centerLS.x();
         extreme.y() = xm;
         extreme.z() = zm;
         extreme2 = secondMap.applyMap(extreme);
-        pre_image = map.applyInverseMap(extreme);
+
+        const Vec3d tmpMinus = extreme2;
+
+        // convert back to voxel space
+        pre_image = frustum.applyInverseMap(extreme2);
         out_min.y() = std::max(y_min, std::min(y_max, pre_image.y()));
 
     } else if (soln_number == 1) {
@@ -287,16 +305,17 @@ calculateBounds<math::NonlinearFrustumMap>(const math::NonlinearFrustumMap& map,
     }
 
     // the near and far
-    // the closest point.  The front of the frustum is at 0
+    // the closest point.  The front of the frustum is at 0 in index space
     double near_dist = std::max(centerLS.z() - radiusLS, 0.);
-    // the farthest point.  The back of the frustum is at mDepth
-    double far_dist = std::min(centerLS.z() + radiusLS, map.getDepth() );
+    // the farthest point.  The back of the frustum is at mDepth in index space
+    double far_dist = std::min(centerLS.z() + radiusLS, frustum.getDepth() );
 
-    Vec3d near_point(0, 0, near_dist);
-    Vec3d far_point(0, 0, far_dist);
+    Vec3d near_point(0.f, 0.f, near_dist);
+    Vec3d far_point(0.f, 0.f, far_dist);
+
+    out_min.z() = std::max(z_min, frustum.applyInverseMap(secondMap.applyMap(near_point)).z());
+    out_max.z() = std::min(z_max, frustum.applyInverseMap(secondMap.applyMap(far_point)).z());
 
-    out_min.z() = std::max(z_min, map.applyInverseMap(secondMap.applyMap(near_point)).z());
-    out_max.z() = std::min(z_max, map.applyInverseMap(secondMap.applyMap(far_point)).z());
 }
 
 } // namespace util
@@ -305,6 +324,6 @@ calculateBounds<math::NonlinearFrustumMap>(const math::NonlinearFrustumMap& map,
 
 #endif // OPENVDB_UTIL_MAPSUTIL_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
